@@ -1,15 +1,14 @@
 __socket = __import__('socket')
 
-__all__     = __socket.__all__
+__all__ = __socket.__all__
 __patched__ = ['fromfd', 'socketpair', 'ssl', 'socket']
 
 from eventlet.patcher import slurp_properties
-slurp_properties(__socket, globals(), 
-    ignore=__patched__, srckeys=dir(__socket))
+slurp_properties(__socket, globals(),
+                 ignore=__patched__, srckeys=dir(__socket))
 
 os = __import__('os')
 import sys
-import warnings
 from eventlet.hubs import get_hub
 from eventlet.greenio import GreenSocket as socket
 from eventlet.greenio import SSL as _SSL  # for exceptions
@@ -18,6 +17,7 @@ from eventlet.greenio import _fileobject
 
 try:
     __original_fromfd__ = __socket.fromfd
+
     def fromfd(*args):
         return socket(__original_fromfd__(*args))
 except AttributeError:
@@ -25,6 +25,7 @@ except AttributeError:
 
 try:
     __original_socketpair__ = __socket.socketpair
+
     def socketpair(*args):
         one, two = __original_socketpair__(*args)
         return socket(one), socket(two)
@@ -32,7 +33,6 @@ except AttributeError:
     pass
 
 
-
 def _convert_to_sslerror(ex):
     """ Transliterates SSL.SysCallErrors to socket.sslerrors"""
     return sslerror((ex.args[0], ex.args[1]))
@@ -41,6 +41,7 @@ def _convert_to_sslerror(ex):
 class GreenSSLObject(object):
     """ Wrapper object around the SSLObjects returned by socket.ssl, which have a
     slightly different interface from SSL.Connection objects. """
+
     def __init__(self, green_ssl_obj):
         """ Should only be called by a 'green' socket.ssl """
         self.connection = green_ssl_obj
@@ -52,7 +53,7 @@ class GreenSSLObject(object):
         else:
             try:
                 self.connection.do_handshake()
-            except _SSL.SysCallError, e:
+            except _SSL.SysCallError as e:
                 raise _convert_to_sslerror(e)
 
     def read(self, n=1024):
@@ -62,7 +63,7 @@ class GreenSSLObject(object):
             return self.connection.read(n)
         except _SSL.ZeroReturnError:
             return ''
-        except _SSL.SysCallError, e:
+        except _SSL.SysCallError as e:
             raise _convert_to_sslerror(e)
 
     def write(self, s):
@@ -70,7 +71,7 @@ class GreenSSLObject(object):
         The return value is the number of bytes written. """
         try:
             return self.connection.write(s)
-        except _SSL.SysCallError, e:
+        except _SSL.SysCallError as e:
             raise _convert_to_sslerror(e)
 
     def server(self):
@@ -84,27 +85,3 @@ class GreenSSLObject(object):
         for debugging purposes; do not parse the content of this string because its
         format can't be parsed unambiguously."""
         return str(self.connection.get_peer_certificate().get_issuer())
-
-
-try:
-    try:
-        # >= Python 2.6
-        from eventlet.green import ssl as ssl_module
-        sslerror = __socket.sslerror
-        __socket.ssl
-        def ssl(sock, certificate=None, private_key=None):
-            warnings.warn("socket.ssl() is deprecated.  Use ssl.wrap_socket() instead.",
-                          DeprecationWarning, stacklevel=2)
-            return ssl_module.sslwrap_simple(sock, private_key, certificate)
-    except ImportError:
-        # <= Python 2.5 compatibility
-        sslerror = __socket.sslerror
-        __socket.ssl
-        def ssl(sock, certificate=None, private_key=None):
-            from eventlet import util
-            wrapped = util.wrap_ssl(sock, certificate, private_key)
-            return GreenSSLObject(wrapped)
-except AttributeError:
-    # if the real socket module doesn't have the ssl method or sslerror
-    # exception, we can't emulate them
-    pass
