@@ -7,12 +7,14 @@ import sys
 import errno
 time = __import__('time')
 
-from eventlet.support import get_errno
-from eventlet.hubs import trampoline
-from eventlet.greenio import set_nonblocking, GreenSocket, SOCKET_CLOSED, CONNECT_ERR, CONNECT_SUCCESS
+from eventlet.support import get_errno, PY33, six
+from eventlet.hubs import trampoline, IOClosed
+from eventlet.greenio import (
+    set_nonblocking, GreenSocket, SOCKET_CLOSED, CONNECT_ERR, CONNECT_SUCCESS,
+)
 orig_socket = __import__('socket')
 socket = orig_socket.socket
-if sys.version_info >= (2,7):
+if sys.version_info >= (2, 7):
     has_ciphers = True
     timeout_exc = SSLError
 else:
@@ -21,45 +23,75 @@ else:
 
 __patched__ = ['SSLSocket', 'wrap_socket', 'sslwrap_simple']
 
-class GreenSSLSocket(__ssl.SSLSocket):
-    """ This is a green version of the SSLSocket class from the ssl module added 
+_original_sslsocket = __ssl.SSLSocket
+
+
+class GreenSSLSocket(_original_sslsocket):
+    """ This is a green version of the SSLSocket class from the ssl module added
     in 2.6.  For documentation on it, please see the Python standard
     documentation.
-    
+
     Python nonblocking ssl objects don't give errors when the other end
     of the socket is closed (they do notice when the other end is shutdown,
     though).  Any write/read operations will simply hang if the socket is
     closed from the other end.  There is no obvious fix for this problem;
     it appears to be a limitation of Python's ssl object implementation.
     A workaround is to set a reasonable timeout on the socket using
-    settimeout(), and to close/reopen the connection when a timeout 
+    settimeout(), and to close/reopen the connection when a timeout
     occurs at an unexpected juncture in the code.
     """
-    # we are inheriting from SSLSocket because its constructor calls 
+    # we are inheriting from SSLSocket because its constructor calls
     # do_handshake whose behavior we wish to override
-    def __init__(self, sock, *args, **kw):
+
+    def __init__(self, sock, keyfile=None, certfile=None,
+                 server_side=False, cert_reqs=CERT_NONE,
+                 ssl_version=PROTOCOL_SSLv23, ca_certs=None,
+                 do_handshake_on_connect=True, *args, **kw):
         if not isinstance(sock, GreenSocket):
             sock = GreenSocket(sock)
 
         self.act_non_blocking = sock.act_non_blocking
-        self._timeout = sock.gettimeout()
-        super(GreenSSLSocket, self).__init__(sock.fd, *args, **kw)
-       
+
+        if six.PY2:
+            # On Python 2 SSLSocket constructor queries the timeout, it'd break without
+            # this assignment
+            self._timeout = sock.gettimeout()
+
+        # nonblocking socket handshaking on connect got disabled so let's pretend it's disabled
+        # even when it's on
+        super(GreenSSLSocket, self).__init__(
+            sock.fd, keyfile, certfile, server_side, cert_reqs, ssl_version,
+            ca_certs, do_handshake_on_connect and six.PY2, *args, **kw)
+
         # the superclass initializer trashes the methods so we remove
         # the local-object versions of them and let the actual class
         # methods shine through
+        # Note: This for Python 2
         try:
             for fn in orig_socket._delegate_methods:
                 delattr(self, fn)
         except AttributeError:
             pass
-       
+
+        if six.PY3:
+            # Python 3 SSLSocket construction process overwrites the timeout so restore it
+            self._timeout = sock.gettimeout()
+
+            # it also sets timeout to None internally apparently (tested with 3.4.2)
+            _original_sslsocket.settimeout(self, 0.0)
+            assert _original_sslsocket.gettimeout(self) == 0.0
+
+            # see note above about handshaking
+            self.do_handshake_on_connect = do_handshake_on_connect
+            if do_handshake_on_connect and self._connected:
+                self.do_handshake()
+
     def settimeout(self, timeout):
         self._timeout = timeout
-        
+
     def gettimeout(self):
         return self._timeout
-    
+
     def setblocking(self, flag):
         if flag:
             self.act_non_blocking = False
@@ -75,7 +107,7 @@ class GreenSSLSocket(__ssl.SSLSocket):
             while True:
                 try:
                     return func(*a, **kw)
-                except SSLError, exc:
+                except SSLError as exc:
                     if get_errno(exc) == SSL_ERROR_WANT_READ:
                         trampoline(self,
                                    read=True,
@@ -95,13 +127,16 @@ class GreenSSLSocket(__ssl.SSLSocket):
         return self._call_trampolining(
             super(GreenSSLSocket, self).write, data)
 
-    def read(self, len=1024):
+    def read(self, *args, **kwargs):
         """Read up to LEN bytes and return them.
         Return zero-length string on EOF."""
-        return self._call_trampolining(
-            super(GreenSSLSocket, self).read, len)
+        try:
+            return self._call_trampolining(
+                super(GreenSSLSocket, self).read, *args, **kwargs)
+        except IOClosed:
+            return b''
 
-    def send (self, data, flags=0):
+    def send(self, data, flags=0):
         if self._sslobj:
             return self._call_trampolining(
                 super(GreenSSLSocket, self).send, data, flags)
@@ -109,7 +144,7 @@ class GreenSSLSocket(__ssl.SSLSocket):
             trampoline(self, write=True, timeout_exc=timeout_exc('timed out'))
             return socket.send(self, data, flags)
 
-    def sendto (self, data, addr, flags=0):
+    def sendto(self, data, addr, flags=0):
         # *NOTE: gross, copied code from ssl.py becase it's not factored well enough to be used as-is
         if self._sslobj:
             raise ValueError("sendto not allowed on instances of %s" %
@@ -118,7 +153,7 @@ class GreenSSLSocket(__ssl.SSLSocket):
             trampoline(self, write=True, timeout_exc=timeout_exc('timed out'))
             return socket.sendto(self, data, addr, flags)
 
-    def sendall (self, data, flags=0):
+    def sendall(self, data, flags=0):
         # *NOTE: gross, copied code from ssl.py becase it's not factored well enough to be used as-is
         if self._sslobj:
             if flags != 0:
@@ -136,12 +171,12 @@ class GreenSSLSocket(__ssl.SSLSocket):
         else:
             while True:
                 try:
-                    return socket.sendall(self, buflen, flags)
-                except orig_socket.error, e:
+                    return socket.sendall(self, data, flags)
+                except orig_socket.error as e:
                     if self.act_non_blocking:
                         raise
                     if get_errno(e) == errno.EWOULDBLOCK:
-                        trampoline(self, write=True, 
+                        trampoline(self, write=True,
                                    timeout=self.gettimeout(), timeout_exc=timeout_exc('timed out'))
                     if get_errno(e) in SOCKET_CLOSED:
                         return ''
@@ -160,37 +195,43 @@ class GreenSSLSocket(__ssl.SSLSocket):
             while True:
                 try:
                     return socket.recv(self, buflen, flags)
-                except orig_socket.error, e:
+                except orig_socket.error as e:
                     if self.act_non_blocking:
                         raise
                     if get_errno(e) == errno.EWOULDBLOCK:
-                        trampoline(self, read=True, 
-                                   timeout=self.gettimeout(), timeout_exc=timeout_exc('timed out'))
+                        try:
+                            trampoline(
+                                self, read=True,
+                                timeout=self.gettimeout(), timeout_exc=timeout_exc('timed out'))
+                        except IOClosed:
+                            return b''
                     if get_errno(e) in SOCKET_CLOSED:
-                        return ''
+                        return b''
                     raise
 
-        
-    def recv_into (self, buffer, nbytes=None, flags=0):
+    def recv_into(self, buffer, nbytes=None, flags=0):
         if not self.act_non_blocking:
-            trampoline(self, read=True, timeout=self.gettimeout(), timeout_exc=timeout_exc('timed out'))
+            trampoline(self, read=True, timeout=self.gettimeout(),
+                       timeout_exc=timeout_exc('timed out'))
         return super(GreenSSLSocket, self).recv_into(buffer, nbytes, flags)
 
-    def recvfrom (self, addr, buflen=1024, flags=0):
+    def recvfrom(self, addr, buflen=1024, flags=0):
         if not self.act_non_blocking:
-            trampoline(self, read=True, timeout=self.gettimeout(), timeout_exc=timeout_exc('timed out'))
+            trampoline(self, read=True, timeout=self.gettimeout(),
+                       timeout_exc=timeout_exc('timed out'))
         return super(GreenSSLSocket, self).recvfrom(addr, buflen, flags)
-        
-    def recvfrom_into (self, buffer, nbytes=None, flags=0):
+
+    def recvfrom_into(self, buffer, nbytes=None, flags=0):
         if not self.act_non_blocking:
-            trampoline(self, read=True, timeout=self.gettimeout(), timeout_exc=timeout_exc('timed out'))
+            trampoline(self, read=True, timeout=self.gettimeout(),
+                       timeout_exc=timeout_exc('timed out'))
         return super(GreenSSLSocket, self).recvfrom_into(buffer, nbytes, flags)
 
     def unwrap(self):
         return GreenSocket(self._call_trampolining(
-                super(GreenSSLSocket, self).unwrap))
+            super(GreenSSLSocket, self).unwrap))
 
-    def do_handshake(self):    
+    def do_handshake(self):
         """Perform a TLS/SSL handshake."""
         return self._call_trampolining(
             super(GreenSSLSocket, self).do_handshake)
@@ -206,7 +247,7 @@ class GreenSSLSocket(__ssl.SSLSocket):
                 while True:
                     try:
                         return real_connect(self, addr)
-                    except orig_socket.error, exc:
+                    except orig_socket.error as exc:
                         if get_errno(exc) in CONNECT_ERR:
                             trampoline(self, write=True)
                         elif get_errno(exc) in CONNECT_SUCCESS:
@@ -218,17 +259,17 @@ class GreenSSLSocket(__ssl.SSLSocket):
                 while True:
                     try:
                         real_connect(self, addr)
-                    except orig_socket.error, exc:
+                    except orig_socket.error as exc:
                         if get_errno(exc) in CONNECT_ERR:
-                            trampoline(self, write=True, 
-                                       timeout=end-time.time(), timeout_exc=timeout_exc('timed out'))
+                            trampoline(
+                                self, write=True,
+                                timeout=end - time.time(), timeout_exc=timeout_exc('timed out'))
                         elif get_errno(exc) in CONNECT_SUCCESS:
                             return
                         else:
                             raise
                     if time.time() >= end:
                         raise timeout_exc('timed out')
-    
 
     def connect(self, addr):
         """Connects to remote ADDR, and then wraps the connection in
@@ -238,14 +279,22 @@ class GreenSSLSocket(__ssl.SSLSocket):
         if self._sslobj:
             raise ValueError("attempt to connect already-connected SSLSocket!")
         self._socket_connect(addr)
-        if has_ciphers:
-            self._sslobj = _ssl.sslwrap(self._sock, False, self.keyfile, self.certfile,
-                                        self.cert_reqs, self.ssl_version,
-                                        self.ca_certs, self.ciphers)
+        server_side = False
+        try:
+            sslwrap = _ssl.sslwrap
+        except AttributeError:
+            # sslwrap was removed in 3.x and later in 2.7.9
+            if six.PY2:
+                sslobj = self._context._wrap_socket(self._sock, server_side, ssl_sock=self)
+            else:
+                context = self.context if PY33 else self._context
+                sslobj = context._wrap_socket(self, server_side)
         else:
-            self._sslobj = _ssl.sslwrap(self._sock, False, self.keyfile, self.certfile,
-                                        self.cert_reqs, self.ssl_version,
-                                        self.ca_certs)
+            sslobj = sslwrap(self._sock, server_side, self.keyfile, self.certfile,
+                             self.cert_reqs, self.ssl_version,
+                             self.ca_certs, *([self.ciphers] if has_ciphers else []))
+
+        self._sslobj = sslobj
         if self.do_handshake_on_connect:
             self.do_handshake()
 
@@ -262,28 +311,30 @@ class GreenSSLSocket(__ssl.SSLSocket):
                     newsock, addr = socket.accept(self)
                     set_nonblocking(newsock)
                     break
-                except orig_socket.error, e:
+                except orig_socket.error as e:
                     if get_errno(e) != errno.EWOULDBLOCK:
                         raise
                     trampoline(self, read=True, timeout=self.gettimeout(),
-                                   timeout_exc=timeout_exc('timed out'))
+                               timeout_exc=timeout_exc('timed out'))
 
-        new_ssl = type(self)(newsock,
-                          keyfile=self.keyfile,
-                          certfile=self.certfile,
-                          server_side=True,
-                          cert_reqs=self.cert_reqs,
-                          ssl_version=self.ssl_version,
-                          ca_certs=self.ca_certs,
-                          do_handshake_on_connect=self.do_handshake_on_connect,
-                          suppress_ragged_eofs=self.suppress_ragged_eofs)
+        new_ssl = type(self)(
+            newsock,
+            keyfile=self.keyfile,
+            certfile=self.certfile,
+            server_side=True,
+            cert_reqs=self.cert_reqs,
+            ssl_version=self.ssl_version,
+            ca_certs=self.ca_certs,
+            do_handshake_on_connect=self.do_handshake_on_connect,
+            suppress_ragged_eofs=self.suppress_ragged_eofs)
         return (new_ssl, addr)
 
     def dup(self):
         raise NotImplementedError("Can't dup an ssl object")
-                               
+
 SSLSocket = GreenSSLSocket
 
+
 def wrap_socket(sock, *a, **kw):
     return GreenSSLSocket(sock, *a, **kw)
 
@@ -294,8 +345,8 @@ if hasattr(__ssl, 'sslwrap_simple'):
         for compability with Python 2.5 and earlier.  Will disappear in
         Python 3.0."""
         ssl_sock = GreenSSLSocket(sock, keyfile=keyfile, certfile=certfile,
-                                  server_side=False, 
-                                  cert_reqs=CERT_NONE, 
-                                  ssl_version=PROTOCOL_SSLv23, 
+                                  server_side=False,
+                                  cert_reqs=CERT_NONE,
+                                  ssl_version=PROTOCOL_SSLv23,
                                   ca_certs=None)
         return ssl_sock
