@@ -1,30 +1,27 @@
-from eventlet.support import get_errno
-from eventlet.hubs import trampoline
-BUFFER_SIZE = 4096
-
-import array
 import errno
 import os
-import socket
 from socket import socket as _original_socket
+import socket
 import sys
 import time
 import warnings
 
+from eventlet.support import get_errno, six
+from eventlet.hubs import trampoline, notify_close, notify_opened, IOClosed
+
 __all__ = ['GreenSocket', 'GreenPipe', 'shutdown_safe']
 
+BUFFER_SIZE = 4096
 CONNECT_ERR = set((errno.EINPROGRESS, errno.EALREADY, errno.EWOULDBLOCK))
 CONNECT_SUCCESS = set((0, errno.EISCONN))
 if sys.platform[:3] == "win":
     CONNECT_ERR.add(errno.WSAEINVAL)   # Bug 67
 
-# Emulate _fileobject class in 3.x implementation
-# Eventually this internal socket structure could be replaced with makefile calls.
-try:
+if six.PY3:
+    from io import IOBase as file
+    _fileobject = socket.SocketIO
+elif six.PY2:
     _fileobject = socket._fileobject
-except AttributeError:
-    def _fileobject(sock, *args, **kwargs):
-        return _original_socket.makefile(sock, *args, **kwargs)
 
 
 def socket_connect(descriptor, address):
@@ -54,7 +51,7 @@ def socket_accept(descriptor):
     """
     try:
         return descriptor.accept()
-    except socket.error, e:
+    except socket.error as e:
         if get_errno(e) == errno.EWOULDBLOCK:
             return None
         raise
@@ -62,12 +59,12 @@ def socket_accept(descriptor):
 
 if sys.platform[:3] == "win":
     # winsock sometimes throws ENOTCONN
-    SOCKET_BLOCKING = set((errno.EWOULDBLOCK,))
+    SOCKET_BLOCKING = set((errno.EAGAIN, errno.EWOULDBLOCK,))
     SOCKET_CLOSED = set((errno.ECONNRESET, errno.ENOTCONN, errno.ESHUTDOWN))
 else:
     # oddly, on linux/darwin, an unconnected socket is expected to block,
     # so we treat ENOTCONN the same as EWOULDBLOCK
-    SOCKET_BLOCKING = set((errno.EWOULDBLOCK, errno.ENOTCONN))
+    SOCKET_BLOCKING = set((errno.EAGAIN, errno.EWOULDBLOCK, errno.ENOTCONN))
     SOCKET_CLOSED = set((errno.ECONNRESET, errno.ESHUTDOWN, errno.EPIPE))
 
 
@@ -121,14 +118,18 @@ class GreenSocket(object):
     Pass False to indicate that socket is already in non-blocking mode
     to save syscalls.
     """
+
+    # This placeholder is to prevent __getattr__ from creating an infinite call loop
+    fd = None
+
     def __init__(self, family_or_realsock=socket.AF_INET, *args, **kwargs):
         should_set_nonblocking = kwargs.pop('set_nonblocking', True)
-        if isinstance(family_or_realsock, (int, long)):
+        if isinstance(family_or_realsock, six.integer_types):
             fd = _original_socket(family_or_realsock, *args, **kwargs)
+            # Notify the hub that this is a newly-opened socket.
+            notify_opened(fd.fileno())
         else:
             fd = family_or_realsock
-            assert not args, args
-            assert not kwargs, kwargs
 
         # import timeout from other socket, if it was there
         try:
@@ -156,21 +157,52 @@ class GreenSocket(object):
         self.listen = fd.listen
         self.setsockopt = fd.setsockopt
         self.shutdown = fd.shutdown
+        self._closed = False
 
     @property
     def _sock(self):
         return self
 
+    if six.PY3:
+        def _get_io_refs(self):
+            return self.fd._io_refs
+
+        def _set_io_refs(self, value):
+            self.fd._io_refs = value
+
+        _io_refs = property(_get_io_refs, _set_io_refs)
+
     # Forward unknown attributes to fd, cache the value for future use.
     # I do not see any simple attribute which could be changed
     # so caching everything in self is fine.
     # If we find such attributes - only attributes having __get__ might be cached.
     # For now - I do not want to complicate it.
     def __getattr__(self, name):
+        if self.fd is None:
+            raise AttributeError(name)
         attr = getattr(self.fd, name)
         setattr(self, name, attr)
         return attr
 
+    def _trampoline(self, fd, read=False, write=False, timeout=None, timeout_exc=None):
+        """ We need to trampoline via the event hub.
+            We catch any signal back from the hub indicating that the operation we
+            were waiting on was associated with a filehandle that's since been
+            invalidated.
+        """
+        if self._closed:
+            # If we did any logging, alerting to a second trampoline attempt on a closed
+            # socket here would be useful.
+            raise IOClosed()
+        try:
+            return trampoline(fd, read=read, write=write, timeout=timeout,
+                              timeout_exc=timeout_exc,
+                              mark_as_closed=self._mark_as_closed)
+        except IOClosed:
+            # This socket's been obsoleted. De-fang it.
+            self._mark_as_closed()
+            raise
+
     def accept(self):
         if self.act_non_blocking:
             return self.fd.accept()
@@ -181,8 +213,18 @@ class GreenSocket(object):
                 client, addr = res
                 set_nonblocking(client)
                 return type(self)(client), addr
-            trampoline(fd, read=True, timeout=self.gettimeout(),
-                timeout_exc=socket.timeout("timed out"))
+            self._trampoline(fd, read=True, timeout=self.gettimeout(),
+                             timeout_exc=socket.timeout("timed out"))
+
+    def _mark_as_closed(self):
+        """ Mark this socket as being closed """
+        self._closed = True
+
+    def __del__(self):
+        # This is in case self.close is not assigned yet (currently the constructor does it)
+        close = getattr(self, 'close', None)
+        if close is not None:
+            close()
 
     def connect(self, address):
         if self.act_non_blocking:
@@ -190,7 +232,10 @@ class GreenSocket(object):
         fd = self.fd
         if self.gettimeout() is None:
             while not socket_connect(fd, address):
-                trampoline(fd, write=True)
+                try:
+                    self._trampoline(fd, write=True)
+                except IOClosed:
+                    raise socket.error(errno.EBADFD)
                 socket_checkerr(fd)
         else:
             end = time.time() + self.gettimeout()
@@ -199,8 +244,12 @@ class GreenSocket(object):
                     return
                 if time.time() >= end:
                     raise socket.timeout("timed out")
-                trampoline(fd, write=True, timeout=end - time.time(),
-                        timeout_exc=socket.timeout("timed out"))
+                try:
+                    self._trampoline(fd, write=True, timeout=end - time.time(),
+                                     timeout_exc=socket.timeout("timed out"))
+                except IOClosed:
+                    # ... we need some workable errno here.
+                    raise socket.error(errno.EBADFD)
                 socket_checkerr(fd)
 
     def connect_ex(self, address):
@@ -210,10 +259,12 @@ class GreenSocket(object):
         if self.gettimeout() is None:
             while not socket_connect(fd, address):
                 try:
-                    trampoline(fd, write=True)
+                    self._trampoline(fd, write=True)
                     socket_checkerr(fd)
-                except socket.error, ex:
+                except socket.error as ex:
                     return get_errno(ex)
+                except IOClosed:
+                    return errno.EBADFD
         else:
             end = time.time() + self.gettimeout()
             while True:
@@ -222,11 +273,13 @@ class GreenSocket(object):
                         return 0
                     if time.time() >= end:
                         raise socket.timeout(errno.EAGAIN)
-                    trampoline(fd, write=True, timeout=end - time.time(),
-                            timeout_exc=socket.timeout(errno.EAGAIN))
+                    self._trampoline(fd, write=True, timeout=end - time.time(),
+                                     timeout_exc=socket.timeout(errno.EAGAIN))
                     socket_checkerr(fd)
-                except socket.error, ex:
+                except socket.error as ex:
                     return get_errno(ex)
+                except IOClosed:
+                    return errno.EBADFD
 
     def dup(self, *args, **kw):
         sock = self.fd.dup(*args, **kw)
@@ -234,12 +287,20 @@ class GreenSocket(object):
         newsock.settimeout(self.gettimeout())
         return newsock
 
-    def makefile(self, *args, **kw):
-        return _fileobject(self.dup(), *args, **kw)
+    if six.PY3:
+        def makefile(self, *args, **kwargs):
+            return _original_socket.makefile(self, *args, **kwargs)
+    else:
+        def makefile(self, *args, **kwargs):
+            dupped = self.dup()
+            res = _fileobject(dupped, *args, **kwargs)
+            if hasattr(dupped, "_drop"):
+                dupped._drop()
+            return res
 
     def makeGreenFile(self, *args, **kw):
         warnings.warn("makeGreenFile has been deprecated, please use "
-            "makefile instead", DeprecationWarning, stacklevel=2)
+                      "makefile instead", DeprecationWarning, stacklevel=2)
         return self.makefile(*args, **kw)
 
     def recv(self, buflen, flags=0):
@@ -249,34 +310,39 @@ class GreenSocket(object):
         while True:
             try:
                 return fd.recv(buflen, flags)
-            except socket.error, e:
+            except socket.error as e:
                 if get_errno(e) in SOCKET_BLOCKING:
                     pass
                 elif get_errno(e) in SOCKET_CLOSED:
                     return ''
                 else:
                     raise
-            trampoline(fd,
-                read=True,
-                timeout=self.gettimeout(),
-                timeout_exc=socket.timeout("timed out"))
+            try:
+                self._trampoline(
+                    fd,
+                    read=True,
+                    timeout=self.gettimeout(),
+                    timeout_exc=socket.timeout("timed out"))
+            except IOClosed as e:
+                # Perhaps we should return '' instead?
+                raise EOFError()
 
     def recvfrom(self, *args):
         if not self.act_non_blocking:
-            trampoline(self.fd, read=True, timeout=self.gettimeout(),
-                    timeout_exc=socket.timeout("timed out"))
+            self._trampoline(self.fd, read=True, timeout=self.gettimeout(),
+                             timeout_exc=socket.timeout("timed out"))
         return self.fd.recvfrom(*args)
 
     def recvfrom_into(self, *args):
         if not self.act_non_blocking:
-            trampoline(self.fd, read=True, timeout=self.gettimeout(),
-                    timeout_exc=socket.timeout("timed out"))
+            self._trampoline(self.fd, read=True, timeout=self.gettimeout(),
+                             timeout_exc=socket.timeout("timed out"))
         return self.fd.recvfrom_into(*args)
 
     def recv_into(self, *args):
         if not self.act_non_blocking:
-            trampoline(self.fd, read=True, timeout=self.gettimeout(),
-                    timeout_exc=socket.timeout("timed out"))
+            self._trampoline(self.fd, read=True, timeout=self.gettimeout(),
+                             timeout_exc=socket.timeout("timed out"))
         return self.fd.recv_into(*args)
 
     def send(self, data, flags=0):
@@ -287,19 +353,21 @@ class GreenSocket(object):
         # blocking socket behavior - sends all, blocks if the buffer is full
         total_sent = 0
         len_data = len(data)
-
         while 1:
             try:
                 total_sent += fd.send(data[total_sent:], flags)
-            except socket.error, e:
+            except socket.error as e:
                 if get_errno(e) not in SOCKET_BLOCKING:
                     raise
 
             if total_sent == len_data:
                 break
 
-            trampoline(self.fd, write=True, timeout=self.gettimeout(),
-                    timeout_exc=socket.timeout("timed out"))
+            try:
+                self._trampoline(self.fd, write=True, timeout=self.gettimeout(),
+                                 timeout_exc=socket.timeout("timed out"))
+            except IOClosed:
+                raise socket.error(errno.ECONNRESET, 'Connection closed by another thread')
 
         return total_sent
 
@@ -310,7 +378,7 @@ class GreenSocket(object):
             tail += self.send(data[tail:], flags)
 
     def sendto(self, *args):
-        trampoline(self.fd, write=True)
+        self._trampoline(self.fd, write=True)
         return self.fd.sendto(*args)
 
     def setblocking(self, flag):
@@ -342,11 +410,41 @@ class GreenSocket(object):
     def gettimeout(self):
         return self._timeout
 
+    if "__pypy__" in sys.builtin_module_names:
+        def _reuse(self):
+            getattr(self.fd, '_sock', self.fd)._reuse()
+
+        def _drop(self):
+            getattr(self.fd, '_sock', self.fd)._drop()
+
 
 class _SocketDuckForFd(object):
-    """ Class implementing all socket method used by _fileobject in cooperative manner using low level os I/O calls."""
+    """Class implementing all socket method used by _fileobject
+    in cooperative manner using low level os I/O calls.
+    """
+    _refcount = 0
+
     def __init__(self, fileno):
         self._fileno = fileno
+        notify_opened(fileno)
+        self._closed = False
+
+    def _trampoline(self, fd, read=False, write=False, timeout=None, timeout_exc=None):
+        if self._closed:
+            # Don't trampoline if we're already closed.
+            raise IOClosed()
+        try:
+            return trampoline(fd, read=read, write=write, timeout=timeout,
+                              timeout_exc=timeout_exc,
+                              mark_as_closed=self._mark_as_closed)
+        except IOClosed:
+            # Our fileno has been obsoleted. Defang ourselves to
+            # prevent spurious closes.
+            self._mark_as_closed()
+            raise
+
+    def _mark_as_closed(self):
+        self._closed = True
 
     @property
     def _sock(self):
@@ -360,10 +458,27 @@ class _SocketDuckForFd(object):
             try:
                 data = os.read(self._fileno, buflen)
                 return data
-            except OSError, e:
-                if get_errno(e) != errno.EAGAIN:
+            except OSError as e:
+                if get_errno(e) not in SOCKET_BLOCKING:
                     raise IOError(*e.args)
-            trampoline(self, read=True)
+            self._trampoline(self, read=True)
+
+    def recv_into(self, buf, nbytes=0, flags=0):
+        if nbytes == 0:
+            nbytes = len(buf)
+        data = self.recv(nbytes)
+        buf[:nbytes] = data
+        return len(data)
+
+    def send(self, data):
+        while True:
+            try:
+                return os.write(self._fileno, data)
+            except OSError as e:
+                if get_errno(e) not in SOCKET_BLOCKING:
+                    raise IOError(*e.args)
+                else:
+                    trampoline(self, write=True)
 
     def sendall(self, data):
         len_data = len(data)
@@ -371,28 +486,44 @@ class _SocketDuckForFd(object):
         fileno = self._fileno
         try:
             total_sent = os_write(fileno, data)
-        except OSError, e:
+        except OSError as e:
             if get_errno(e) != errno.EAGAIN:
                 raise IOError(*e.args)
             total_sent = 0
         while total_sent < len_data:
-            trampoline(self, write=True)
+            self._trampoline(self, write=True)
             try:
                 total_sent += os_write(fileno, data[total_sent:])
-            except OSError, e:
+            except OSError as e:
                 if get_errno(e) != errno. EAGAIN:
                     raise IOError(*e.args)
 
     def __del__(self):
+        self._close()
+
+    def _close(self):
+        notify_close(self._fileno)
+        self._mark_as_closed()
         try:
             os.close(self._fileno)
         except:
-            # os.close may fail if __init__ didn't complete (i.e file dscriptor passed to popen was invalid
+            # os.close may fail if __init__ didn't complete
+            # (i.e file dscriptor passed to popen was invalid
             pass
 
     def __repr__(self):
         return "%s:%d" % (self.__class__.__name__, self._fileno)
 
+    def _reuse(self):
+        self._refcount += 1
+
+    def _drop(self):
+        self._refcount -= 1
+        if self._refcount == 0:
+            self._close()
+    # Python3
+    _decref_socketios = _drop
+
 
 def _operationOnClosedFile(*args, **kwargs):
     raise ValueError("I/O operation on closed file")
@@ -410,11 +541,12 @@ class GreenPipe(_fileobject):
     - Universal new lines are not supported and newlines property not implementeded
     - file argument can be descriptor, file name or file object.
     """
+
     def __init__(self, f, mode='r', bufsize=-1):
-        if not isinstance(f, (basestring, int, file)):
+        if not isinstance(f, six.string_types + (int, file)):
             raise TypeError('f(ile) should be int, str, unicode or file, not %r' % f)
 
-        if isinstance(f, basestring):
+        if isinstance(f, six.string_types):
             f = open(f, mode, 0)
 
         if isinstance(f, int):
@@ -428,7 +560,7 @@ class GreenPipe(_fileobject):
             self._name = f.name
             f.close()
 
-        super(GreenPipe, self).__init__(_SocketDuckForFd(fileno), mode, bufsize)
+        super(GreenPipe, self).__init__(_SocketDuckForFd(fileno), mode)
         set_nonblocking(self)
         self.softspace = 0
 
@@ -446,34 +578,20 @@ class GreenPipe(_fileobject):
 
     def close(self):
         super(GreenPipe, self).close()
-        for method in ['fileno', 'flush', 'isatty', 'next', 'read', 'readinto',
-                   'readline', 'readlines', 'seek', 'tell', 'truncate',
-                   'write', 'xreadlines', '__iter__', 'writelines']:
+        for method in [
+                'fileno', 'flush', 'isatty', 'next', 'read', 'readinto',
+                'readline', 'readlines', 'seek', 'tell', 'truncate',
+                'write', 'xreadlines', '__iter__', '__next__', 'writelines']:
             setattr(self, method, _operationOnClosedFile)
 
-    if getattr(file, '__enter__', None):
-        def __enter__(self):
-            return self
+    def __enter__(self):
+        return self
 
-        def __exit__(self, *args):
-            self.close()
-
-    def readinto(self, buf):
-        data = self.read(len(buf)) # FIXME could it be done without allocating intermediate?
-        n = len(data)
-        try:
-            buf[:n] = data
-        except TypeError, err:
-            if not isinstance(buf, array.array):
-                raise err
-            buf[:n] = array.array('c', data)
-        return n
+    def __exit__(self, *args):
+        self.close()
 
     def _get_readahead_len(self):
-        try:
-            return len(self._rbuf.getvalue()) # StringIO in 2.5
-        except AttributeError:
-            return len(self._rbuf) # str in 2.4
+        return len(self._rbuf.getvalue())
 
     def _clear_readahead_buf(self):
         len = self._get_readahead_len()
@@ -484,40 +602,40 @@ class GreenPipe(_fileobject):
         self.flush()
         try:
             return os.lseek(self.fileno(), 0, 1) - self._get_readahead_len()
-        except OSError, e:
+        except OSError as e:
             raise IOError(*e.args)
 
     def seek(self, offset, whence=0):
         self.flush()
-        if whence == 1 and offset == 0: # tell synonym
+        if whence == 1 and offset == 0:  # tell synonym
             return self.tell()
-        if whence == 1: # adjust offset by what is read ahead
+        if whence == 1:  # adjust offset by what is read ahead
             offset -= self._get_readahead_len()
         try:
             rv = os.lseek(self.fileno(), offset, whence)
-        except OSError, e:
+        except OSError as e:
             raise IOError(*e.args)
         else:
             self._clear_readahead_buf()
             return rv
 
-    if getattr(file, "truncate", None): # not all OSes implement truncate
+    if getattr(file, "truncate", None):  # not all OSes implement truncate
         def truncate(self, size=-1):
             self.flush()
             if size == -1:
                 size = self.tell()
             try:
                 rv = os.ftruncate(self.fileno(), size)
-            except OSError, e:
+            except OSError as e:
                 raise IOError(*e.args)
             else:
-                self.seek(size) # move position&clear buffer
+                self.seek(size)  # move position&clear buffer
                 return rv
 
     def isatty(self):
         try:
             return os.isatty(self.fileno())
-        except OSError, e:
+        except OSError as e:
             raise IOError(*e.args)
 
 
@@ -527,16 +645,16 @@ try:
 except ImportError:
     # pyOpenSSL not installed, define exceptions anyway for convenience
     class SSL(object):
-        class WantWriteError(object):
+        class WantWriteError(Exception):
             pass
 
-        class WantReadError(object):
+        class WantReadError(Exception):
             pass
 
-        class ZeroReturnError(object):
+        class ZeroReturnError(Exception):
             pass
 
-        class SysCallError(object):
+        class SysCallError(Exception):
             pass
 
 
@@ -556,7 +674,7 @@ def shutdown_safe(sock):
         except TypeError:
             # SSL.Connection
             return sock.shutdown()
-    except socket.error, e:
+    except socket.error as e:
         # we don't care if the socket is already closed;
         # this will often be the case in an http server context
         if get_errno(e) != errno.ENOTCONN:
