@@ -1,14 +1,19 @@
-import sys
 import imp
+import sys
+
+from eventlet.support import six
+
 
 __all__ = ['inject', 'import_patched', 'monkey_patch', 'is_monkey_patched']
 
 __exclude = set(('__builtins__', '__file__', '__name__'))
 
+
 class SysModulesSaver(object):
     """Class that captures some subset of the current state of
     sys.modules.  Pass in an iterator of module names to the
     constructor."""
+
     def __init__(self, module_names=()):
         self._saved = {}
         imp.acquire_lock()
@@ -24,7 +29,7 @@ class SysModulesSaver(object):
         sys.modules.
         """
         try:
-            for modname, mod in self._saved.iteritems():
+            for modname, mod in six.iteritems(self._saved):
                 if mod is not None:
                     sys.modules[modname] = mod
                 else:
@@ -34,7 +39,7 @@ class SysModulesSaver(object):
                         pass
         finally:
             imp.release_lock()
-                
+
 
 def inject(module_name, new_globals, *additional_modules):
     """Base method for "injecting" greened modules into an imported module.  It
@@ -42,6 +47,10 @@ def inject(module_name, new_globals, *additional_modules):
     that the already-imported modules in *additional_modules* are used when
     *module_name* makes its imports.
 
+    **Note:** This function does not create or change any sys.modules item, so
+    if your greened module use code like 'sys.modules["your_module_name"]', you
+    need to update sys.modules by yourself.
+
     *new_globals* is either None or a globals dictionary that gets populated
     with the contents of the *module_name* module.  This is useful when creating
     a "green" version of some other module.
@@ -64,9 +73,9 @@ def inject(module_name, new_globals, *additional_modules):
             _green_select_modules() +
             _green_socket_modules() +
             _green_thread_modules() +
-            _green_time_modules()) 
-            #_green_MySQLdb()) # enable this after a short baking-in period
-    
+            _green_time_modules())
+        # _green_MySQLdb()) # enable this after a short baking-in period
+
     # after this we are gonna screw with sys.modules, so capture the
     # state of all the modules we're going to mess with, and lock
     saver = SysModulesSaver([name for name, m in additional_modules])
@@ -77,22 +86,22 @@ def inject(module_name, new_globals, *additional_modules):
     for name, mod in additional_modules:
         sys.modules[name] = mod
 
-    ## Remove the old module from sys.modules and reimport it while
-    ## the specified modules are in place
+    # Remove the old module from sys.modules and reimport it while
+    # the specified modules are in place
     sys.modules.pop(module_name, None)
     try:
         module = __import__(module_name, {}, {}, module_name.split('.')[:-1])
 
         if new_globals is not None:
-            ## Update the given globals dictionary with everything from this new module
+            # Update the given globals dictionary with everything from this new module
             for name in dir(module):
                 if name not in __exclude:
                     new_globals[name] = getattr(module, name)
 
-        ## Keep a reference to the new module to prevent it from dying
+        # Keep a reference to the new module to prevent it from dying
         sys.modules[patched_name] = module
     finally:
-        saver.restore()  ## Put the original modules back
+        saver.restore()  # Put the original modules back
 
     return module
 
@@ -122,7 +131,7 @@ def patch_function(func, *additional_modules):
             _green_os_modules() +
             _green_select_modules() +
             _green_socket_modules() +
-            _green_thread_modules() + 
+            _green_thread_modules() +
             _green_time_modules())
 
     def patched(*args, **kw):
@@ -136,6 +145,7 @@ def patch_function(func, *additional_modules):
             saver.restore()
     return patched
 
+
 def _original_patch_function(func, *module_names):
     """Kind of the contrapositive of patch_function: decorates a
     function such that when it's called, sys.modules is populated only
@@ -155,7 +165,7 @@ def _original_patch_function(func, *module_names):
 
 
 def original(modname):
-    """ This returns an unpatched version of a module; this is useful for 
+    """ This returns an unpatched version of a module; this is useful for
     Eventlet itself (i.e. tpool)."""
     # note that it's not necessary to temporarily install unpatched
     # versions of all patchable modules during the import of the
@@ -172,19 +182,22 @@ def original(modname):
     # some rudimentary dependency checking -- fortunately the modules
     # we're working on don't have many dependencies so we can just do
     # some special-casing here
-    deps = {'threading':'thread', 'Queue':'threading'}
+    if six.PY2:
+        deps = {'threading': 'thread', 'Queue': 'threading'}
+    if six.PY3:
+        deps = {'threading': '_thread', 'queue': 'threading'}
     if modname in deps:
         dependency = deps[modname]
         saver.save(dependency)
         sys.modules[dependency] = original(dependency)
     try:
         real_mod = __import__(modname, {}, {}, modname.split('.')[:-1])
-        if modname == 'Queue' and not hasattr(real_mod, '_threading'):
+        if modname in ('Queue', 'queue') and not hasattr(real_mod, '_threading'):
             # tricky hack: Queue's constructor in <2.7 imports
             # threading on every instantiation; therefore we wrap
             # it so that it always gets the original threading
             real_mod.Queue.__init__ = _original_patch_function(
-                real_mod.Queue.__init__, 
+                real_mod.Queue.__init__,
                 'threading')
         # save a reference to the unpatched module so it doesn't get lost
         sys.modules[original_name] = real_mod
@@ -194,34 +207,38 @@ def original(modname):
     return sys.modules[original_name]
 
 already_patched = {}
+
+
 def monkey_patch(**on):
     """Globally patches certain system modules to be greenthread-friendly.
 
     The keyword arguments afford some control over which modules are patched.
     If no keyword arguments are supplied, all possible modules are patched.
     If keywords are set to True, only the specified modules are patched.  E.g.,
-    ``monkey_patch(socket=True, select=True)`` patches only the select and 
-    socket modules.  Most arguments patch the single module of the same name 
-    (os, time, select).  The exceptions are socket, which also patches the ssl 
+    ``monkey_patch(socket=True, select=True)`` patches only the select and
+    socket modules.  Most arguments patch the single module of the same name
+    (os, time, select).  The exceptions are socket, which also patches the ssl
     module if present; and thread, which patches thread, threading, and Queue.
 
     It's safe to call monkey_patch multiple times.
-    """    
-    accepted_args = set(('os', 'select', 'socket', 
-                         'thread', 'time', 'psycopg', 'MySQLdb'))
-    default_on = on.pop("all",None)
-    for k in on.iterkeys():
+    """
+    accepted_args = set(('os', 'select', 'socket',
+                         'thread', 'time', 'psycopg', 'MySQLdb', '__builtin__'))
+    default_on = on.pop("all", None)
+    for k in six.iterkeys(on):
         if k not in accepted_args:
-            raise TypeError("monkey_patch() got an unexpected "\
-                                "keyword argument %r" % k)
+            raise TypeError("monkey_patch() got an unexpected "
+                            "keyword argument %r" % k)
     if default_on is None:
         default_on = not (True in on.values())
     for modname in accepted_args:
         if modname == 'MySQLdb':
             # MySQLdb is only on when explicitly patched for the moment
             on.setdefault(modname, False)
+        if modname == '__builtin__':
+            on.setdefault(modname, False)
         on.setdefault(modname, default_on)
-        
+
     modules_to_patch = []
     if on['os'] and not already_patched.get('os'):
         modules_to_patch += _green_os_modules()
@@ -241,6 +258,9 @@ def monkey_patch(**on):
     if on.get('MySQLdb') and not already_patched.get('MySQLdb'):
         modules_to_patch += _green_MySQLdb()
         already_patched['MySQLdb'] = True
+    if on.get('__builtin__') and not already_patched.get('__builtin__'):
+        modules_to_patch += _green_builtins()
+        already_patched['__builtin__'] = True
     if on['psycopg'] and not already_patched.get('psycopg'):
         try:
             from eventlet.support import psycopg2_patcher
@@ -267,6 +287,13 @@ def monkey_patch(**on):
     finally:
         imp.release_lock()
 
+    if sys.version_info >= (3, 3):
+        import importlib._bootstrap
+        thread = original('_thread')
+        # importlib must use real thread locks, not eventlet.Semaphore
+        importlib._bootstrap._thread = thread
+
+
 def is_monkey_patched(module):
     """Returns True if the given module is monkeypatched currently, False if
     not.  *module* can be either the module itself or its name.
@@ -276,16 +303,19 @@ def is_monkey_patched(module):
     import_patched), this might not be correct about that particular
     module."""
     return module in already_patched or \
-           getattr(module, '__name__', None) in already_patched
+        getattr(module, '__name__', None) in already_patched
+
 
 def _green_os_modules():
     from eventlet.green import os
     return [('os', os)]
 
+
 def _green_select_modules():
     from eventlet.green import select
     return [('select', select)]
 
+
 def _green_socket_modules():
     from eventlet.green import socket
     try:
@@ -294,16 +324,22 @@ def _green_socket_modules():
     except ImportError:
         return [('socket', socket)]
 
+
 def _green_thread_modules():
     from eventlet.green import Queue
     from eventlet.green import thread
     from eventlet.green import threading
-    return [('Queue', Queue), ('thread', thread), ('threading', threading)]
+    if six.PY2:
+        return [('Queue', Queue), ('thread', thread), ('threading', threading)]
+    if six.PY3:
+        return [('queue', Queue), ('_thread', thread), ('threading', threading)]
+
 
 def _green_time_modules():
     from eventlet.green import time
     return [('time', time)]
 
+
 def _green_MySQLdb():
     try:
         from eventlet.green import MySQLdb
@@ -312,26 +348,34 @@ def _green_MySQLdb():
         return []
 
 
+def _green_builtins():
+    try:
+        from eventlet.green import builtin
+        return [('__builtin__', builtin)]
+    except ImportError:
+        return []
+
+
 def slurp_properties(source, destination, ignore=[], srckeys=None):
     """Copy properties from *source* (assumed to be a module) to
     *destination* (assumed to be a dict).
-    
+
     *ignore* lists properties that should not be thusly copied.
-    *srckeys* is a list of keys to copy, if the source's __all__ is 
+    *srckeys* is a list of keys to copy, if the source's __all__ is
     untrustworthy.
     """
     if srckeys is None:
         srckeys = source.__all__
-    destination.update(dict([(name, getattr(source, name))
-                              for name in srckeys
-                              if not (
-                                name.startswith('__') or 
-                                name in ignore)
-                            ]))
+    destination.update(dict([
+        (name, getattr(source, name))
+        for name in srckeys
+        if not (name.startswith('__') or name in ignore)
+    ]))
 
 
 if __name__ == "__main__":
-    import sys
     sys.argv.pop(0)
     monkey_patch()
-    execfile(sys.argv[0])
+    with open(sys.argv[0]) as f:
+        code = compile(f.read(), sys.argv[0], 'exec')
+        exec(code)
