@@ -5,9 +5,9 @@ try:
     getcurrent = greenlet.greenlet.getcurrent
     GreenletExit = greenlet.greenlet.GreenletExit
     preserves_excinfo = (distutils.version.LooseVersion(greenlet.__version__)
-            >= distutils.version.LooseVersion('0.3.2'))
+                         >= distutils.version.LooseVersion('0.3.2'))
     greenlet = greenlet.greenlet
-except ImportError, e:
+except ImportError as e:
     raise
     try:
         from py.magic import greenlet
@@ -24,6 +24,6 @@ except ImportError, e:
             try:
                 from support.stacklesss import greenlet, getcurrent, GreenletExit
                 preserves_excinfo = False
-                (greenlet, getcurrent, GreenletExit) # silence pyflakes
-            except ImportError, e:
+                (greenlet, getcurrent, GreenletExit)  # silence pyflakes
+            except ImportError as e:
                 raise ImportError("Unable to find an implementation of greenlet.")
