@@ -6,12 +6,14 @@ import traceback
 import types
 import warnings
 
-from eventlet.green import urllib
-from eventlet.green import socket
 from eventlet.green import BaseHTTPServer
-from eventlet import greenpool
+from eventlet.green import socket
+from eventlet.green import urllib
 from eventlet import greenio
-from eventlet.support import get_errno
+from eventlet import greenpool
+from eventlet import support
+from eventlet.support import six
+
 
 DEFAULT_MAX_SIMULTANEOUS_REQUESTS = 1024
 DEFAULT_MAX_HTTP_VERSION = 'HTTP/1.1'
@@ -20,17 +22,19 @@ MAX_HEADER_LINE = 8192
 MAX_TOTAL_HEADER_SIZE = 65536
 MINIMUM_CHUNK_SIZE = 4096
 # %(client_port)s is also available
-DEFAULT_LOG_FORMAT= ('%(client_ip)s - - [%(date_time)s] "%(request_line)s"'
-                     ' %(status_code)s %(body_length)s %(wall_seconds).6f')
+DEFAULT_LOG_FORMAT = ('%(client_ip)s - - [%(date_time)s] "%(request_line)s"'
+                      ' %(status_code)s %(body_length)s %(wall_seconds).6f')
+is_accepting = True
 
 __all__ = ['server', 'format_date_time']
 
 # Weekday and month names for HTTP date/time formatting; always English!
 _weekdayname = ["Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"]
-_monthname = [None, # Dummy so we can use 1-based month numbers
+_monthname = [None,  # Dummy so we can use 1-based month numbers
               "Jan", "Feb", "Mar", "Apr", "May", "Jun",
               "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]
 
+
 def format_date_time(timestamp):
     """Formats a unix timestamp into an HTTP standard string."""
     year, month, day, hh, mm, ss, wd, _y, _z = time.gmtime(timestamp)
@@ -38,11 +42,13 @@ def format_date_time(timestamp):
         _weekdayname[wd], day, _monthname[month], year, hh, mm, ss
     )
 
+
 # Collections of error codes to compare against.  Not all attributes are set
 # on errno module on all platforms, so some are literals :(
 BAD_SOCK = set((errno.EBADF, 10053))
 BROKEN_SOCK = set((errno.EPIPE, errno.ECONNRESET))
 
+
 # special flag return value for apps
 class _AlreadyHandled(object):
 
@@ -52,17 +58,24 @@ class _AlreadyHandled(object):
     def next(self):
         raise StopIteration
 
+    __next__ = next
+
+
 ALREADY_HANDLED = _AlreadyHandled()
 
+
 class Input(object):
+
     def __init__(self,
                  rfile,
                  content_length,
+                 sock,
                  wfile=None,
                  wfile_line=None,
                  chunked_input=False):
 
         self.rfile = rfile
+        self._sock = sock
         if content_length is not None:
             content_length = int(content_length)
         self.content_length = content_length
@@ -74,37 +87,61 @@ class Input(object):
         self.chunked_input = chunked_input
         self.chunk_length = -1
 
-    def _do_read(self, reader, length=None):
-        if self.wfile is not None:
-            ## 100 Continue
-            self.wfile.write(self.wfile_line)
-            self.wfile = None
-            self.wfile_line = None
+        # (optional) headers to send with a "100 Continue" response. Set by
+        # calling set_hundred_continue_respose_headers() on env['wsgi.input']
+        self.hundred_continue_headers = None
+        self.is_hundred_continue_response_sent = False
 
+    def send_hundred_continue_response(self):
+        towrite = []
+
+        # 100 Continue status line
+        towrite.append(self.wfile_line)
+
+        # Optional headers
+        if self.hundred_continue_headers is not None:
+            # 100 Continue headers
+            for header in self.hundred_continue_headers:
+                towrite.append(six.b('%s: %s\r\n' % header))
+
+        # Blank line
+        towrite.append(b'\r\n')
+
+        self.wfile.writelines(towrite)
+
+        # Reinitialize chunk_length (expect more data)
+        self.chunk_length = -1
+
+    def _do_read(self, reader, length=None):
+        if self.wfile is not None and \
+                not self.is_hundred_continue_response_sent:
+            # 100 Continue response
+            self.send_hundred_continue_response()
+            self.is_hundred_continue_response_sent = True
         if length is None and self.content_length is not None:
             length = self.content_length - self.position
         if length and length > self.content_length - self.position:
             length = self.content_length - self.position
         if not length:
-            return ''
+            return b''
         try:
             read = reader(length)
         except greenio.SSL.ZeroReturnError:
-            read = ''
+            read = b''
         self.position += len(read)
         return read
 
     def _chunked_read(self, rfile, length=None, use_readline=False):
-        if self.wfile is not None:
-            ## 100 Continue
-            self.wfile.write(self.wfile_line)
-            self.wfile = None
-            self.wfile_line = None
+        if self.wfile is not None and \
+                not self.is_hundred_continue_response_sent:
+            # 100 Continue response
+            self.send_hundred_continue_response()
+            self.is_hundred_continue_response_sent = True
         try:
             if length == 0:
                 return ""
 
-            if length < 0:
+            if length and length < 0:
                 length = None
 
             if use_readline:
@@ -138,13 +175,13 @@ class Input(object):
                     if use_readline and data[-1] == "\n":
                         break
                 else:
-                    self.chunk_length = int(rfile.readline().split(";", 1)[0], 16)
+                    self.chunk_length = int(rfile.readline().split(b";", 1)[0], 16)
                     self.position = 0
                     if self.chunk_length == 0:
                         rfile.readline()
         except greenio.SSL.ZeroReturnError:
             pass
-        return ''.join(response)
+        return b''.join(response)
 
     def read(self, length=None):
         if self.chunked_input:
@@ -161,10 +198,22 @@ class Input(object):
         return self._do_read(self.rfile.readlines, hint)
 
     def __iter__(self):
-        return iter(self.read())
+        return iter(self.read, b'')
 
     def get_socket(self):
-        return self.rfile._sock
+        return self._sock
+
+    def set_hundred_continue_response_headers(self, headers,
+                                              capitalize_response_headers=True):
+        # Response headers capitalization (default)
+        # CONTent-TYpe: TExt/PlaiN -> Content-Type: TExt/PlaiN
+        # Per HTTP RFC standard, header name is case-insensitive.
+        # Please, fix your client to ignore header case if possible.
+        if capitalize_response_headers:
+            headers = [
+                ('-'.join([x.capitalize() for x in key.split('-')]), value)
+                for key, value in headers]
+        self.hundred_continue_headers = headers
 
 
 class HeaderLineTooLong(Exception):
@@ -186,7 +235,7 @@ class FileObjectForHeaders(object):
         if size < 0:
             sz = MAX_HEADER_LINE
         rv = self.fp.readline(sz)
-        if size < 0 and len(rv) >= MAX_HEADER_LINE:
+        if len(rv) >= MAX_HEADER_LINE:
             raise HeaderLineTooLong()
         self.total_header_size += len(rv)
         if self.total_header_size > MAX_TOTAL_HEADER_SIZE:
@@ -197,6 +246,7 @@ class FileObjectForHeaders(object):
 class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
     protocol_version = 'HTTP/1.1'
     minimum_chunk_size = MINIMUM_CHUNK_SIZE
+    capitalize_response_headers = True
 
     def setup(self):
         # overriding SocketServer.setup to correctly handle SSL.Connection objects
@@ -211,7 +261,7 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
                 self.wfile = socket._fileobject(conn, "wb", self.wbufsize)
             else:
                 # it's a SSLObject, or a martian
-                raise NotImplementedError("wsgi.py doesn't support sockets "\
+                raise NotImplementedError("wsgi.py doesn't support sockets "
                                           "of type %s" % type(conn))
 
     def handle_one_request(self):
@@ -226,14 +276,14 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
             self.raw_requestline = self.rfile.readline(self.server.url_length_limit)
             if len(self.raw_requestline) == self.server.url_length_limit:
                 self.wfile.write(
-                    "HTTP/1.0 414 Request URI Too Long\r\n"
-                    "Connection: close\r\nContent-length: 0\r\n\r\n")
+                    b"HTTP/1.0 414 Request URI Too Long\r\n"
+                    b"Connection: close\r\nContent-length: 0\r\n\r\n")
                 self.close_connection = 1
                 return
         except greenio.SSL.ZeroReturnError:
             self.raw_requestline = ''
-        except socket.error, e:
-            if get_errno(e) not in BAD_SOCK:
+        except socket.error as e:
+            if support.get_errno(e) not in BAD_SOCK:
                 raise
             self.raw_requestline = ''
 
@@ -248,27 +298,27 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
                 return
         except HeaderLineTooLong:
             self.wfile.write(
-                "HTTP/1.0 400 Header Line Too Long\r\n"
-                "Connection: close\r\nContent-length: 0\r\n\r\n")
+                b"HTTP/1.0 400 Header Line Too Long\r\n"
+                b"Connection: close\r\nContent-length: 0\r\n\r\n")
             self.close_connection = 1
             return
         except HeadersTooLarge:
             self.wfile.write(
-                "HTTP/1.0 400 Headers Too Large\r\n"
-                "Connection: close\r\nContent-length: 0\r\n\r\n")
+                b"HTTP/1.0 400 Headers Too Large\r\n"
+                b"Connection: close\r\nContent-length: 0\r\n\r\n")
             self.close_connection = 1
             return
         finally:
             self.rfile = orig_rfile
 
-        content_length = self.headers.getheader('content-length')
+        content_length = self.headers.get('content-length')
         if content_length:
             try:
                 int(content_length)
             except ValueError:
                 self.wfile.write(
-                    "HTTP/1.0 400 Bad Request\r\n"
-                    "Connection: close\r\nContent-length: 0\r\n\r\n")
+                    b"HTTP/1.0 400 Bad Request\r\n"
+                    b"Connection: close\r\nContent-length: 0\r\n\r\n")
                 self.close_connection = 1
                 return
 
@@ -278,9 +328,9 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
             self.server.outstanding_requests += 1
             try:
                 self.handle_one_response()
-            except socket.error, e:
+            except socket.error as e:
                 # Broken pipe, connection reset by peer
-                if get_errno(e) not in BROKEN_SOCK:
+                if support.get_errno(e) not in BROKEN_SOCK:
                     raise
         finally:
             self.server.outstanding_requests -= 1
@@ -304,62 +354,65 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
                 status, response_headers = headers_set
                 headers_sent.append(1)
                 header_list = [header[0].lower() for header in response_headers]
-                towrite.append('%s %s\r\n' % (self.protocol_version, status))
+                towrite.append(six.b('%s %s\r\n' % (self.protocol_version, status)))
                 for header in response_headers:
-                    towrite.append('%s: %s\r\n' % header)
+                    towrite.append(six.b('%s: %s\r\n' % header))
 
                 # send Date header?
                 if 'date' not in header_list:
-                    towrite.append('Date: %s\r\n' % (format_date_time(time.time()),))
+                    towrite.append(six.b('Date: %s\r\n' % (format_date_time(time.time()),)))
 
                 client_conn = self.headers.get('Connection', '').lower()
                 send_keep_alive = False
                 if self.close_connection == 0 and \
-                   self.server.keepalive and (client_conn == 'keep-alive' or \
-                    (self.request_version == 'HTTP/1.1' and
-                     not client_conn == 'close')):
+                   self.server.keepalive and (client_conn == 'keep-alive' or
+                                              (self.request_version == 'HTTP/1.1' and
+                                               not client_conn == 'close')):
                         # only send keep-alives back to clients that sent them,
                         # it's redundant for 1.1 connections
-                        send_keep_alive = (client_conn == 'keep-alive')
-                        self.close_connection = 0
+                    send_keep_alive = (client_conn == 'keep-alive')
+                    self.close_connection = 0
                 else:
                     self.close_connection = 1
 
                 if 'content-length' not in header_list:
                     if self.request_version == 'HTTP/1.1':
                         use_chunked[0] = True
-                        towrite.append('Transfer-Encoding: chunked\r\n')
+                        towrite.append(b'Transfer-Encoding: chunked\r\n')
                     elif 'content-length' not in header_list:
                         # client is 1.0 and therefore must read to EOF
                         self.close_connection = 1
 
                 if self.close_connection:
-                    towrite.append('Connection: close\r\n')
+                    towrite.append(b'Connection: close\r\n')
                 elif send_keep_alive:
-                    towrite.append('Connection: keep-alive\r\n')
-                towrite.append('\r\n')
+                    towrite.append(b'Connection: keep-alive\r\n')
+                towrite.append(b'\r\n')
                 # end of header writing
 
             if use_chunked[0]:
-                ## Write the chunked encoding
-                towrite.append("%x\r\n%s\r\n" % (len(data), data))
+                # Write the chunked encoding
+                towrite.append(six.b("%x" % (len(data),)) + b"\r\n" + data + b"\r\n")
             else:
                 towrite.append(data)
             try:
                 _writelines(towrite)
                 length[0] = length[0] + sum(map(len, towrite))
             except UnicodeEncodeError:
-                self.server.log_message("Encountered non-ascii unicode while attempting to write wsgi response: %r" % [x for x in towrite if isinstance(x, unicode)])
+                self.server.log_message(
+                    "Encountered non-ascii unicode while attempting to write"
+                    "wsgi response: %r" %
+                    [x for x in towrite if isinstance(x, six.text_type)])
                 self.server.log_message(traceback.format_exc())
                 _writelines(
                     ["HTTP/1.1 500 Internal Server Error\r\n",
-                    "Connection: close\r\n",
-                    "Content-type: text/plain\r\n",
-                    "Content-length: 98\r\n",
-                    "Date: %s\r\n" % format_date_time(time.time()),
-                    "\r\n",
-                    ("Internal Server Error: wsgi application passed "
-                     "a unicode object to the server instead of a string.")])
+                     "Connection: close\r\n",
+                     "Content-type: text/plain\r\n",
+                     "Content-length: 98\r\n",
+                     "Date: %s\r\n" % format_date_time(time.time()),
+                     "\r\n",
+                     ("Internal Server Error: wsgi application passed "
+                      "a unicode object to the server instead of a string.")])
 
         def start_response(status, response_headers, exc_info=None):
             status_code[0] = status.split()[0]
@@ -367,52 +420,60 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
                 try:
                     if headers_sent:
                         # Re-raise original exception if headers sent
-                        raise exc_info[0], exc_info[1], exc_info[2]
+                        six.reraise(exc_info[0], exc_info[1], exc_info[2])
                 finally:
                     # Avoid dangling circular ref
                     exc_info = None
 
-            capitalized_headers = [('-'.join([x.capitalize()
-                                              for x in key.split('-')]), value)
-                                   for key, value in response_headers]
+            # Response headers capitalization
+            # CONTent-TYpe: TExt/PlaiN -> Content-Type: TExt/PlaiN
+            # Per HTTP RFC standard, header name is case-insensitive.
+            # Please, fix your client to ignore header case if possible.
+            if self.capitalize_response_headers:
+                response_headers = [
+                    ('-'.join([x.capitalize() for x in key.split('-')]), value)
+                    for key, value in response_headers]
 
-            headers_set[:] = [status, capitalized_headers]
+            headers_set[:] = [status, response_headers]
             return write
 
         try:
             try:
                 result = self.application(self.environ, start_response)
                 if (isinstance(result, _AlreadyHandled)
-                    or isinstance(getattr(result, '_obj', None), _AlreadyHandled)):
+                        or isinstance(getattr(result, '_obj', None), _AlreadyHandled)):
                     self.close_connection = 1
                     return
+
+                # Set content-length if possible
                 if not headers_sent and hasattr(result, '__len__') and \
                         'Content-Length' not in [h for h, _v in headers_set[1]]:
                     headers_set[1].append(('Content-Length', str(sum(map(len, result)))))
+
                 towrite = []
                 towrite_size = 0
                 just_written_size = 0
+                minimum_write_chunk_size = int(self.environ.get(
+                    'eventlet.minimum_write_chunk_size', self.minimum_chunk_size))
                 for data in result:
                     towrite.append(data)
                     towrite_size += len(data)
-                    if towrite_size >= self.minimum_chunk_size:
-                        write(''.join(towrite))
+                    if towrite_size >= minimum_write_chunk_size:
+                        write(b''.join(towrite))
                         towrite = []
                         just_written_size = towrite_size
                         towrite_size = 0
                 if towrite:
                     just_written_size = towrite_size
-                    write(''.join(towrite))
+                    write(b''.join(towrite))
                 if not headers_sent or (use_chunked[0] and just_written_size):
-                    write('')
+                    write(b'')
             except Exception:
                 self.close_connection = 1
                 tb = traceback.format_exc()
                 self.server.log_message(tb)
                 if not headers_set:
-                    err_body = ""
-                    if(self.server.debug):
-                        err_body = tb
+                    err_body = six.b(tb) if self.server.debug else b''
                     start_response("500 Internal Server Error",
                                    [('Content-type', 'text/plain'),
                                     ('Content-length', len(err_body))])
@@ -421,9 +482,9 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
             if hasattr(result, 'close'):
                 result.close()
             if (self.environ['eventlet.input'].chunked_input or
-                    self.environ['eventlet.input'].position \
-                    < self.environ['eventlet.input'].content_length):
-                ## Read and discard body if there was no pending 100-continue
+                    self.environ['eventlet.input'].position
+                    < (self.environ['eventlet.input'].content_length or 0)):
+                # Read and discard body if there was no pending 100-continue
                 if not self.environ['eventlet.input'].wfile:
                     # NOTE: MINIMUM_CHUNK_SIZE is used here for purpose different than chunking.
                     # We use it only cause it's at hand and has reasonable value in terms of
@@ -465,12 +526,15 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
         if len(pq) > 1:
             env['QUERY_STRING'] = pq[1]
 
-        if self.headers.typeheader is None:
-            env['CONTENT_TYPE'] = self.headers.type
-        else:
-            env['CONTENT_TYPE'] = self.headers.typeheader
+        ct = self.headers.get('content-type')
+        if ct is None:
+            try:
+                ct = self.headers.type
+            except AttributeError:
+                ct = self.headers.get_content_type()
+        env['CONTENT_TYPE'] = ct
 
-        length = self.headers.getheader('content-length')
+        length = self.headers.get('content-length')
         if length:
             env['CONTENT_LENGTH'] = length
         env['SERVER_PROTOCOL'] = 'HTTP/1.0'
@@ -482,8 +546,14 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
         env['REMOTE_PORT'] = str(self.client_address[1])
         env['GATEWAY_INTERFACE'] = 'CGI/1.1'
 
-        for h in self.headers.headers:
-            k, v = h.split(':', 1)
+        try:
+            headers = self.headers.headers
+        except AttributeError:
+            headers = self.headers._headers
+        else:
+            headers = [h.split(':', 1) for h in headers]
+
+        for k, v in headers:
             k = k.replace('-', '_').upper()
             v = v.strip()
             if k in env:
@@ -496,13 +566,13 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
 
         if env.get('HTTP_EXPECT') == '100-continue':
             wfile = self.wfile
-            wfile_line = 'HTTP/1.1 100 Continue\r\n\r\n'
+            wfile_line = b'HTTP/1.1 100 Continue\r\n'
         else:
             wfile = None
             wfile_line = None
         chunked = env.get('HTTP_TRANSFER_ENCODING', '').lower() == 'chunked'
         env['wsgi.input'] = env['eventlet.input'] = Input(
-            self.rfile, length, wfile=wfile, wfile_line=wfile_line,
+            self.rfile, length, self.connection, wfile=wfile, wfile_line=wfile_line,
             chunked_input=chunked)
         env['eventlet.posthooks'] = []
 
@@ -511,16 +581,19 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
     def finish(self):
         try:
             BaseHTTPServer.BaseHTTPRequestHandler.finish(self)
-        except socket.error, e:
+        except socket.error as e:
             # Broken pipe, connection reset by peer
-            if get_errno(e) not in BROKEN_SOCK:
+            if support.get_errno(e) not in BROKEN_SOCK:
                 raise
         greenio.shutdown_safe(self.connection)
         self.connection.close()
 
+    def handle_expect_100(self):
+        return True
 
 
 class Server(BaseHTTPServer.HTTPServer):
+
     def __init__(self,
                  socket,
                  address,
@@ -535,7 +608,9 @@ class Server(BaseHTTPServer.HTTPServer):
                  log_output=True,
                  log_format=DEFAULT_LOG_FORMAT,
                  url_length_limit=MAX_REQUEST_LINE,
-                 debug=True):
+                 debug=True,
+                 socket_timeout=None,
+                 capitalize_response_headers=True):
 
         self.outstanding_requests = 0
         self.socket = socket
@@ -556,6 +631,15 @@ class Server(BaseHTTPServer.HTTPServer):
         self.log_format = log_format
         self.url_length_limit = url_length_limit
         self.debug = debug
+        self.socket_timeout = socket_timeout
+        self.capitalize_response_headers = capitalize_response_headers
+
+        if not self.capitalize_response_headers:
+            warnings.warn("""capitalize_response_headers is disabled.
+ Please, make sure you know what you are doing.
+ HTTP headers names are case-insensitive per RFC standard.
+ Most likely, you need to fix HTTP parsing in your client software.""",
+                          DeprecationWarning, stacklevel=3)
 
     def get_environ(self):
         d = {
@@ -574,18 +658,34 @@ class Server(BaseHTTPServer.HTTPServer):
             d.update(self.environ)
         return d
 
-    def process_request(self, (socket, address)):
+    def process_request(self, sock_params):
         # The actual request handling takes place in __init__, so we need to
         # set minimum_chunk_size before __init__ executes and we don't want to modify
         # class variable
-        proto = types.InstanceType(self.protocol)
+        sock, address = sock_params
+        proto = new(self.protocol)
         if self.minimum_chunk_size is not None:
             proto.minimum_chunk_size = self.minimum_chunk_size
-        proto.__init__(socket, address, self)
+        proto.capitalize_response_headers = self.capitalize_response_headers
+        try:
+            proto.__init__(sock, address, self)
+        except socket.timeout:
+            # Expected exceptions are not exceptional
+            sock.close()
+            if self.debug:
+                # similar to logging "accepted" in server()
+                self.log_message('(%s) timed out %r' % (self.pid, address))
 
     def log_message(self, message):
         self.log.write(message + '\n')
 
+
+try:
+    new = types.InstanceType
+except AttributeError:
+    new = lambda cls: cls.__new__(cls)
+
+
 try:
     import ssl
     ACCEPT_EXCEPTIONS = (socket.error, ssl.SSLError)
@@ -595,6 +695,7 @@ except ImportError:
     ACCEPT_EXCEPTIONS = (socket.error,)
     ACCEPT_ERRNO = set((errno.EPIPE, errno.EBADF, errno.ECONNRESET))
 
+
 def server(sock, site,
            log=None,
            environ=None,
@@ -609,28 +710,61 @@ def server(sock, site,
            log_output=True,
            log_format=DEFAULT_LOG_FORMAT,
            url_length_limit=MAX_REQUEST_LINE,
-           debug=True):
-    """  Start up a wsgi server handling requests from the supplied server
-    socket.  This function loops forever.  The *sock* object will be closed after server exits,
-    but the underlying file descriptor will remain open, so if you have a dup() of *sock*,
-    it will remain usable.
+           debug=True,
+           socket_timeout=None,
+           capitalize_response_headers=True):
+    """Start up a WSGI server handling requests from the supplied server
+    socket.  This function loops forever.  The *sock* object will be
+    closed after server exits, but the underlying file descriptor will
+    remain open, so if you have a dup() of *sock*, it will remain usable.
+
+    .. warning::
+
+        At the moment :func:`server` will always wait for active connections to finish before
+        exiting, even if there's an exception raised inside it
+        (*all* exceptions are handled the same way, including :class:`greenlet.GreenletExit`
+        and those inheriting from `BaseException`).
+
+        While this may not be an issue normally, when it comes to long running HTTP connections
+        (like :mod:`eventlet.websocket`) it will become problematic and calling
+        :meth:`~eventlet.greenthread.GreenThread.wait` on a thread that runs the server may hang,
+        even after using :meth:`~eventlet.greenthread.GreenThread.kill`, as long
+        as there are active connections.
 
     :param sock: Server socket, must be already bound to a port and listening.
     :param site: WSGI application function.
-    :param log: File-like object that logs should be written to.  If not specified, sys.stderr is used.
+    :param log: File-like object that logs should be written to.
+                If not specified, sys.stderr is used.
     :param environ: Additional parameters that go into the environ dictionary of every request.
     :param max_size: Maximum number of client connections opened at any time by this server.
-    :param max_http_version: Set to "HTTP/1.0" to make the server pretend it only supports HTTP 1.0.  This can help with applications or clients that don't behave properly using HTTP 1.1.
+    :param max_http_version: Set to "HTTP/1.0" to make the server pretend it only supports HTTP 1.0.
+                This can help with applications or clients that don't behave properly using HTTP 1.1.
     :param protocol: Protocol class.  Deprecated.
     :param server_event: Used to collect the Server object.  Deprecated.
-    :param minimum_chunk_size: Minimum size in bytes for http chunks.  This  can be used to improve performance of applications which yield many small strings, though using it technically violates the WSGI spec.
-    :param log_x_forwarded_for: If True (the default), logs the contents of the x-forwarded-for header in addition to the actual client ip address in the 'client_ip' field of the log line.
-    :param custom_pool: A custom GreenPool instance which is used to spawn client green threads.  If this is supplied, max_size is ignored.
-    :param keepalive: If set to False, disables keepalives on the server; all connections will be closed after serving one request.
+    :param minimum_chunk_size: Minimum size in bytes for http chunks.  This can be used to improve
+                performance of applications which yield many small strings, though
+                using it technically violates the WSGI spec. This can be overridden
+                on a per request basis by setting environ['eventlet.minimum_write_chunk_size'].
+    :param log_x_forwarded_for: If True (the default), logs the contents of the x-forwarded-for
+                header in addition to the actual client ip address in the 'client_ip' field of the
+                log line.
+    :param custom_pool: A custom GreenPool instance which is used to spawn client green threads.
+                If this is supplied, max_size is ignored.
+    :param keepalive: If set to False, disables keepalives on the server; all connections will be
+                closed after serving one request.
     :param log_output: A Boolean indicating if the server will log data or not.
-    :param log_format: A python format string that is used as the template to generate log lines.  The following values can be formatted into it: client_ip, date_time, request_line, status_code, body_length, wall_seconds.  The default is a good example of how to use it.
-    :param url_length_limit: A maximum allowed length of the request url. If exceeded, 414 error is returned.
-    :param debug: True if the server should send exception tracebacks to the clients on 500 errors.  If False, the server will respond with empty bodies.
+    :param log_format: A python format string that is used as the template to generate log lines.
+                The following values can be formatted into it: client_ip, date_time, request_line,
+                status_code, body_length, wall_seconds.  The default is a good example of how to
+                use it.
+    :param url_length_limit: A maximum allowed length of the request url. If exceeded, 414 error
+                is returned.
+    :param debug: True if the server should send exception tracebacks to the clients on 500 errors.
+                If False, the server will respond with empty bodies.
+    :param socket_timeout: Timeout for client connections' socket operations. Default None means
+                wait forever.
+    :param capitalize_response_headers: Normalize response headers' names to Foo-Bar.
+                Default is True.
     """
     serv = Server(sock, sock.getsockname(),
                   site, log,
@@ -643,7 +777,10 @@ def server(sock, site,
                   log_output=log_output,
                   log_format=log_format,
                   url_length_limit=url_length_limit,
-                  debug=debug)
+                  debug=debug,
+                  socket_timeout=socket_timeout,
+                  capitalize_response_headers=capitalize_response_headers,
+                  )
     if server_event is not None:
         server_event.send(serv)
     if max_size is None:
@@ -666,27 +803,31 @@ def server(sock, site,
 
         serv.log.write("(%s) wsgi starting up on %s://%s%s/\n" % (
             serv.pid, scheme, host, port))
-        while True:
+        while is_accepting:
             try:
                 client_socket = sock.accept()
+                client_socket[0].settimeout(serv.socket_timeout)
                 if debug:
                     serv.log.write("(%s) accepted %r\n" % (
                         serv.pid, client_socket[1]))
                 try:
                     pool.spawn_n(serv.process_request, client_socket)
                 except AttributeError:
-                    warnings.warn("wsgi's pool should be an instance of " \
-                        "eventlet.greenpool.GreenPool, is %s. Please convert your"\
-                        " call site to use GreenPool instead" % type(pool),
-                        DeprecationWarning, stacklevel=2)
+                    warnings.warn("wsgi's pool should be an instance of "
+                                  "eventlet.greenpool.GreenPool, is %s. Please convert your"
+                                  " call site to use GreenPool instead" % type(pool),
+                                  DeprecationWarning, stacklevel=2)
                     pool.execute_async(serv.process_request, client_socket)
-            except ACCEPT_EXCEPTIONS, e:
-                if get_errno(e) not in ACCEPT_ERRNO:
+            except ACCEPT_EXCEPTIONS as e:
+                if support.get_errno(e) not in ACCEPT_ERRNO:
                     raise
             except (KeyboardInterrupt, SystemExit):
                 serv.log.write("wsgi exiting\n")
                 break
     finally:
+        pool.waitall()
+        serv.log.write("(%s) wsgi exited, is_accepting=%s\n" % (
+            serv.pid, is_accepting))
         try:
             # NOTE: It's not clear whether we want this to leave the
             # socket open or close it.  Use cases like Spawning want
@@ -694,6 +835,6 @@ def server(sock, site,
             # that far we might as well not bother closing sock at
             # all.
             sock.close()
-        except socket.error, e:
-            if get_errno(e) not in BROKEN_SOCK:
+        except socket.error as e:
+            if support.get_errno(e) not in BROKEN_SOCK:
                 traceback.print_exc()
