@@ -1,27 +1,33 @@
 # package is named tests, not test, so it won't be confused with test in stdlib
+from __future__ import print_function
+
 import errno
+import gc
 import os
-import resource
+try:
+    import resource
+except ImportError:
+    resource = None
 import signal
+import subprocess
+import sys
 import unittest
 import warnings
 
 import eventlet
-from eventlet import debug, hubs
+from eventlet import tpool
 
 
 # convenience for importers
 main = unittest.main
 
-def s2b(s):
-    """portable way to convert string to bytes. In 3.x socket.send and recv require bytes"""
-    return s.encode()
 
 def skipped(func):
     """ Decorator that marks a function as skipped.  Uses nose's SkipTest exception
     if installed.  Without nose, this will count skipped tests as passing tests."""
     try:
         from nose.plugins.skip import SkipTest
+
         def skipme(*a, **k):
             raise SkipTest()
         skipme.__name__ = func.__name__
@@ -29,7 +35,7 @@ def skipped(func):
     except ImportError:
         # no nose, we'll just skip the test ourselves
         def skipme(*a, **k):
-            print "Skipping", func.__name__
+            print(("Skipping {0}".format(func.__name__)))
         skipme.__name__ = func.__name__
         return skipme
 
@@ -76,17 +82,6 @@ def skip_unless(condition):
     return skipped_wrapper
 
 
-def requires_twisted(func):
-    """ Decorator that skips a test if Twisted is not present."""
-    def requirement(_f):
-        from eventlet.hubs import get_hub
-        try:
-            return 'Twisted' in type(get_hub()).__name__
-        except Exception:
-            return False
-    return skip_unless(requirement)(func)
-
-
 def using_pyevent(_f):
     from eventlet.hubs import get_hub
     return 'pyevent' in type(get_hub()).__module__
@@ -99,9 +94,9 @@ def skip_with_pyevent(func):
 
 def skip_on_windows(func):
     """ Decorator that skips a test on Windows."""
-    import sys
     return skip_if(sys.platform.startswith('win'))(func)
 
+
 def skip_if_no_itimer(func):
     """ Decorator that skips a test if the `itimer` module isn't found """
     has_itimer = False
@@ -173,34 +168,30 @@ class LimitedTestCase(unittest.TestCase):
             signal.signal(signal.SIGALRM, self.previous_alarm[0])
             signal.alarm(self.previous_alarm[1])
 
-        try:
-            hub = hubs.get_hub()
-            num_readers = len(hub.get_readers())
-            num_writers = len(hub.get_writers())
-            assert num_readers == num_writers == 0
-        except AssertionError:
-            print "ERROR: Hub not empty"
-            print debug.format_hub_timers()
-            print debug.format_hub_listeners()
+        tpool.killall()
+        gc.collect()
+        eventlet.sleep(0)
+        verify_hub_empty()
 
-    def assert_less_than(self, a,b,msg=None):
-        if msg:
-            self.assert_(a<b, msg)
-        else:
-            self.assert_(a<b, "%s not less than %s" % (a,b))
+    def assert_less_than(self, a, b, msg=None):
+        msg = msg or "%s not less than %s" % (a, b)
+        assert a < b, msg
 
     assertLessThan = assert_less_than
 
-    def assert_less_than_equal(self, a,b,msg=None):
-        if msg:
-            self.assert_(a<=b, msg)
-        else:
-            self.assert_(a<=b, "%s not less than or equal to %s" % (a,b))
+    def assert_less_than_equal(self, a, b, msg=None):
+        msg = msg or "%s not less than or equal to %s" % (a, b)
+        assert a <= b, msg
 
     assertLessThanEqual = assert_less_than_equal
 
 
 def check_idle_cpu_usage(duration, allowed_part):
+    if resource is None:
+        # TODO: use https://code.google.com/p/psutil/
+        from nose.plugins.skip import SkipTest
+        raise SkipTest('CPU usage testing not supported (`import resource` failed)')
+
     r1 = resource.getrusage(resource.RUSAGE_SELF)
     eventlet.sleep(duration)
     r2 = resource.getrusage(resource.RUSAGE_SELF)
@@ -218,7 +209,8 @@ def verify_hub_empty():
     num_readers = len(hub.get_readers())
     num_writers = len(hub.get_writers())
     num_timers = hub.get_timers_count()
-    assert num_readers == 0 and num_writers == 0, "Readers: %s Writers: %s" % (num_readers, num_writers)
+    assert num_readers == 0 and num_writers == 0, "Readers: %s Writers: %s" % (
+        num_readers, num_writers)
 
 
 def find_command(command):
@@ -228,6 +220,7 @@ def find_command(command):
             return p
     raise IOError(errno.ENOENT, 'Command not found: %r' % command)
 
+
 def silence_warnings(func):
     def wrapper(*args, **kw):
         warnings.simplefilter('ignore', DeprecationWarning)
@@ -253,15 +246,17 @@ def get_database_auth():
     connect function.
     """
     import os
-    retval = {'MySQLdb':{'host': 'localhost','user': 'root','passwd': ''},
-              'psycopg2':{'user':'test'}}
+    retval = {
+        'MySQLdb': {'host': 'localhost', 'user': 'root', 'passwd': ''},
+        'psycopg2': {'user': 'test'},
+    }
     try:
         import json
     except ImportError:
         try:
             import simplejson as json
         except ImportError:
-            print "No json implementation, using baked-in db credentials."
+            print("No json implementation, using baked-in db credentials.")
             return retval
 
     if 'EVENTLET_DB_TEST_AUTH' in os.environ:
@@ -275,12 +270,32 @@ def get_database_auth():
             # Have to convert unicode objects to str objects because
             # mysqldb is dum. Using a doubly-nested list comprehension
             # because we know that the structure is a two-level dict.
-            return dict([(str(modname), dict([(str(k), str(v))
-                                       for k, v in connectargs.items()]))
-                         for modname, connectargs in auth_utf8.items()])
+            return dict(
+                [(str(modname), dict(
+                    [(str(k), str(v)) for k, v in connectargs.items()]))
+                 for modname, connectargs in auth_utf8.items()])
         except IOError:
             pass
     return retval
 
+
+def run_python(path):
+    if not path.endswith('.py'):
+        path += '.py'
+    path = os.path.abspath(path)
+    dir_ = os.path.dirname(path)
+    new_env = os.environ.copy()
+    new_env['PYTHONPATH'] = os.pathsep.join(sys.path + [dir_])
+    p = subprocess.Popen(
+        [sys.executable, path],
+        env=new_env,
+        stderr=subprocess.STDOUT,
+        stdin=subprocess.PIPE,
+        stdout=subprocess.PIPE,
+    )
+    output, _ = p.communicate()
+    return output
+
+
 certificate_file = os.path.join(os.path.dirname(__file__), 'test_server.crt')
 private_key_file = os.path.join(os.path.dirname(__file__), 'test_server.key')
