@@ -1,17 +1,22 @@
-"Test cases for db_pool"
+'''Test cases for db_pool
+'''
+from __future__ import print_function
+
 import sys
 import os
 import traceback
 from unittest import TestCase, main
 
-from tests import skipped, skip_unless, skip_with_pyevent, get_database_auth
+from tests import mock, skipped, skip_unless, skip_with_pyevent, get_database_auth
 from eventlet import event
 from eventlet import db_pool
+from eventlet.support import six
 import eventlet
 
 
 class DBTester(object):
     __test__ = False  # so that nose doesn't try to execute this directly
+
     def setUp(self):
         self.create_db()
         self.connection = None
@@ -23,6 +28,7 @@ class DBTester(object):
         );""")
         connection.commit()
         cursor.close()
+        connection.close()
 
     def tearDown(self):
         if self.connection:
@@ -53,6 +59,7 @@ class Mock(object):
 
 class DBConnectionPool(DBTester):
     __test__ = False  # so that nose doesn't try to execute this directly
+
     def setUp(self):
         super(DBConnectionPool, self).setUp()
         self.pool = self.create_pool()
@@ -67,10 +74,10 @@ class DBConnectionPool(DBTester):
     def assert_cursor_works(self, cursor):
         cursor.execute("select 1")
         rows = cursor.fetchall()
-        self.assert_(rows)
+        assert rows
 
     def test_connecting(self):
-        self.assert_(self.connection is not None)
+        assert self.connection is not None
 
     def test_create_cursor(self):
         cursor = self.connection.cursor()
@@ -85,7 +92,7 @@ class DBConnectionPool(DBTester):
         cursor = self.connection.cursor()
         try:
             cursor.execute("garbage blah blah")
-            self.assert_(False)
+            assert False
         except AssertionError:
             raise
         except Exception:
@@ -94,43 +101,43 @@ class DBConnectionPool(DBTester):
 
     def test_put_none(self):
         # the pool is of size 1, and its only connection is out
-        self.assert_(self.pool.free() == 0)
+        assert self.pool.free() == 0
         self.pool.put(None)
         # ha ha we fooled it into thinking that we had a dead process
-        self.assert_(self.pool.free() == 1)
+        assert self.pool.free() == 1
         conn2 = self.pool.get()
-        self.assert_(conn2 is not None)
-        self.assert_(conn2.cursor)
+        assert conn2 is not None
+        assert conn2.cursor
         self.pool.put(conn2)
 
     def test_close_does_a_put(self):
-        self.assert_(self.pool.free() == 0)
+        assert self.pool.free() == 0
         self.connection.close()
-        self.assert_(self.pool.free() == 1)
+        assert self.pool.free() == 1
         self.assertRaises(AttributeError, self.connection.cursor)
 
     @skipped
     def test_deletion_does_a_put(self):
         # doing a put on del causes some issues if __del__ is called in the
         # main coroutine, so, not doing that for now
-        self.assert_(self.pool.free() == 0)
+        assert self.pool.free() == 0
         self.connection = None
-        self.assert_(self.pool.free() == 1)
+        assert self.pool.free() == 1
 
     def test_put_doesnt_double_wrap(self):
         self.pool.put(self.connection)
         conn = self.pool.get()
-        self.assert_(not isinstance(conn._base, db_pool.PooledConnectionWrapper))
+        assert not isinstance(conn._base, db_pool.PooledConnectionWrapper)
         self.pool.put(conn)
 
     def test_bool(self):
-        self.assert_(self.connection)
+        assert self.connection
         self.connection.close()
-        self.assert_(not self.connection)
+        assert not self.connection
 
     def fill_up_table(self, conn):
         curs = conn.cursor()
-        for i in range(1000):
+        for i in six.moves.range(1000):
             curs.execute('insert into test_table (value_int) values (%s)' % i)
         conn.commit()
 
@@ -143,6 +150,7 @@ class DBConnectionPool(DBTester):
         results = []
         SHORT_QUERY = "select * from test_table"
         evt = event.Event()
+
         def a_query():
             self.assert_cursor_works(curs)
             curs.execute(SHORT_QUERY)
@@ -176,7 +184,7 @@ class DBConnectionPool(DBTester):
         self.pool.put(conn2)
 
     def test_visibility_from_other_connections(self):
-        self.pool = self.create_pool(3)
+        self.pool = self.create_pool(max_size=3)
         conn = self.pool.get()
         conn2 = self.pool.get()
         curs = conn.cursor()
@@ -209,7 +217,7 @@ class DBConnectionPool(DBTester):
     def test_two_simultaneous_connections(self):
         # timing-sensitive test, disabled until we come up with a better
         # way to do this
-        self.pool = self.create_pool(2)
+        self.pool = self.create_pool(max_size=2)
         conn = self.pool.get()
         self.set_up_dummy_table(conn)
         self.fill_up_table(conn)
@@ -223,12 +231,14 @@ class DBConnectionPool(DBTester):
         SHORT_QUERY = "select * from test_table where row_id <= 20"
 
         evt = event.Event()
+
         def long_running_query():
             self.assert_cursor_works(curs)
             curs.execute(LONG_QUERY)
             results.append(1)
             evt.send()
         evt2 = event.Event()
+
         def short_running_query():
             self.assert_cursor_works(curs2)
             curs2.execute(SHORT_QUERY)
@@ -258,33 +268,35 @@ class DBConnectionPool(DBTester):
         self.assert_(isinstance(self.connection,
                                 db_pool.GenericConnectionWrapper))
         conn = self.pool._unwrap_connection(self.connection)
-        self.assert_(not isinstance(conn, db_pool.GenericConnectionWrapper))
+        assert not isinstance(conn, db_pool.GenericConnectionWrapper)
 
-        self.assertEquals(None, self.pool._unwrap_connection(None))
-        self.assertEquals(None, self.pool._unwrap_connection(1))
+        self.assertEqual(None, self.pool._unwrap_connection(None))
+        self.assertEqual(None, self.pool._unwrap_connection(1))
 
         # testing duck typing here -- as long as the connection has a
         # _base attribute, it should be unwrappable
         x = Mock()
         x._base = 'hi'
-        self.assertEquals('hi', self.pool._unwrap_connection(x))
+        self.assertEqual('hi', self.pool._unwrap_connection(x))
         conn.close()
 
     def test_safe_close(self):
         self.pool._safe_close(self.connection, quiet=True)
-        self.assertEquals(len(self.pool.free_items), 1)
+        self.assertEqual(len(self.pool.free_items), 1)
 
         self.pool._safe_close(None)
         self.pool._safe_close(1)
 
         # now we're really going for 100% coverage
         x = Mock()
+
         def fail():
             raise KeyboardInterrupt()
         x.close = fail
         self.assertRaises(KeyboardInterrupt, self.pool._safe_close, x)
 
         x = Mock()
+
         def fail2():
             raise RuntimeError("if this line has been printed, the test succeeded")
         x.close = fail2
@@ -296,7 +308,7 @@ class DBConnectionPool(DBTester):
         self.pool = self.create_pool(max_size=2, max_idle=0)
         self.connection = self.pool.get()
         self.connection.close()
-        self.assertEquals(len(self.pool.free_items), 0)
+        self.assertEqual(len(self.pool.free_items), 0)
 
     def test_zero_max_age(self):
         self.pool.put(self.connection)
@@ -304,7 +316,7 @@ class DBConnectionPool(DBTester):
         self.pool = self.create_pool(max_size=2, max_age=0)
         self.connection = self.pool.get()
         self.connection.close()
-        self.assertEquals(len(self.pool.free_items), 0)
+        self.assertEqual(len(self.pool.free_items), 0)
 
     @skipped
     def test_max_idle(self):
@@ -315,19 +327,19 @@ class DBConnectionPool(DBTester):
         self.pool = self.create_pool(max_size=2, max_idle=0.02)
         self.connection = self.pool.get()
         self.connection.close()
-        self.assertEquals(len(self.pool.free_items), 1)
+        self.assertEqual(len(self.pool.free_items), 1)
         eventlet.sleep(0.01)  # not long enough to trigger the idle timeout
-        self.assertEquals(len(self.pool.free_items), 1)
+        self.assertEqual(len(self.pool.free_items), 1)
         self.connection = self.pool.get()
         self.connection.close()
-        self.assertEquals(len(self.pool.free_items), 1)
+        self.assertEqual(len(self.pool.free_items), 1)
         eventlet.sleep(0.01)  # idle timeout should have fired but done nothing
-        self.assertEquals(len(self.pool.free_items), 1)
+        self.assertEqual(len(self.pool.free_items), 1)
         self.connection = self.pool.get()
         self.connection.close()
-        self.assertEquals(len(self.pool.free_items), 1)
-        eventlet.sleep(0.03) # long enough to trigger idle timeout for real
-        self.assertEquals(len(self.pool.free_items), 0)
+        self.assertEqual(len(self.pool.free_items), 1)
+        eventlet.sleep(0.03)  # long enough to trigger idle timeout for real
+        self.assertEqual(len(self.pool.free_items), 0)
 
     @skipped
     def test_max_idle_many(self):
@@ -339,11 +351,11 @@ class DBConnectionPool(DBTester):
         self.connection, conn2 = self.pool.get(), self.pool.get()
         self.connection.close()
         eventlet.sleep(0.01)
-        self.assertEquals(len(self.pool.free_items), 1)
+        self.assertEqual(len(self.pool.free_items), 1)
         conn2.close()
-        self.assertEquals(len(self.pool.free_items), 2)
+        self.assertEqual(len(self.pool.free_items), 2)
         eventlet.sleep(0.02)  # trigger cleanup of conn1 but not conn2
-        self.assertEquals(len(self.pool.free_items), 1)
+        self.assertEqual(len(self.pool.free_items), 1)
 
     @skipped
     def test_max_age(self):
@@ -354,14 +366,14 @@ class DBConnectionPool(DBTester):
         self.pool = self.create_pool(max_size=2, max_age=0.05)
         self.connection = self.pool.get()
         self.connection.close()
-        self.assertEquals(len(self.pool.free_items), 1)
+        self.assertEqual(len(self.pool.free_items), 1)
         eventlet.sleep(0.01)  # not long enough to trigger the age timeout
-        self.assertEquals(len(self.pool.free_items), 1)
+        self.assertEqual(len(self.pool.free_items), 1)
         self.connection = self.pool.get()
         self.connection.close()
-        self.assertEquals(len(self.pool.free_items), 1)
-        eventlet.sleep(0.05) # long enough to trigger age timeout
-        self.assertEquals(len(self.pool.free_items), 0)
+        self.assertEqual(len(self.pool.free_items), 1)
+        eventlet.sleep(0.05)  # long enough to trigger age timeout
+        self.assertEqual(len(self.pool.free_items), 0)
 
     @skipped
     def test_max_age_many(self):
@@ -372,13 +384,13 @@ class DBConnectionPool(DBTester):
         self.pool = self.create_pool(max_size=2, max_age=0.15)
         self.connection, conn2 = self.pool.get(), self.pool.get()
         self.connection.close()
-        self.assertEquals(len(self.pool.free_items), 1)
+        self.assertEqual(len(self.pool.free_items), 1)
         eventlet.sleep(0)  # not long enough to trigger the age timeout
-        self.assertEquals(len(self.pool.free_items), 1)
-        eventlet.sleep(0.2) # long enough to trigger age timeout
-        self.assertEquals(len(self.pool.free_items), 0)
+        self.assertEqual(len(self.pool.free_items), 1)
+        eventlet.sleep(0.2)  # long enough to trigger age timeout
+        self.assertEqual(len(self.pool.free_items), 0)
         conn2.close()  # should not be added to the free items
-        self.assertEquals(len(self.pool.free_items), 0)
+        self.assertEqual(len(self.pool.free_items), 0)
 
     def test_waiters_get_woken(self):
         # verify that when there's someone waiting on an empty pool
@@ -389,23 +401,24 @@ class DBConnectionPool(DBTester):
         self.pool = self.create_pool(max_size=1, max_age=0)
 
         self.connection = self.pool.get()
-        self.assertEquals(self.pool.free(), 0)
-        self.assertEquals(self.pool.waiting(), 0)
+        self.assertEqual(self.pool.free(), 0)
+        self.assertEqual(self.pool.waiting(), 0)
         e = event.Event()
+
         def retrieve(pool, ev):
             c = pool.get()
             ev.send(c)
         eventlet.spawn(retrieve, self.pool, e)
-        eventlet.sleep(0) # these two sleeps should advance the retrieve
-        eventlet.sleep(0) # coroutine until it's waiting in get()
-        self.assertEquals(self.pool.free(), 0)
-        self.assertEquals(self.pool.waiting(), 1)
+        eventlet.sleep(0)  # these two sleeps should advance the retrieve
+        eventlet.sleep(0)  # coroutine until it's waiting in get()
+        self.assertEqual(self.pool.free(), 0)
+        self.assertEqual(self.pool.waiting(), 1)
         self.pool.put(self.connection)
         timer = eventlet.Timeout(1)
         conn = e.wait()
         timer.cancel()
-        self.assertEquals(self.pool.free(), 0)
-        self.assertEquals(self.pool.waiting(), 0)
+        self.assertEqual(self.pool.free(), 0)
+        self.assertEqual(self.pool.waiting(), 0)
         self.pool.put(conn)
 
     @skipped
@@ -415,31 +428,33 @@ class DBConnectionPool(DBTester):
         iterations = 20000
         c = self.connection.cursor()
         self.connection.commit()
+
         def bench(c):
-            for i in xrange(iterations):
+            for i in six.moves.range(iterations):
                 c.execute('select 1')
 
         bench(c)  # warm-up
         results = []
-        for i in xrange(3):
+        for i in range(3):
             start = time.time()
             bench(c)
             end = time.time()
-            results.append(end-start)
+            results.append(end - start)
 
-        print "\n%u iterations took an average of %f seconds, (%s) in %s\n" % (
-            iterations, sum(results)/len(results), results, type(self))
+        print("\n%u iterations took an average of %f seconds, (%s) in %s\n" % (
+            iterations, sum(results) / len(results), results, type(self)))
 
     def test_raising_create(self):
         # if the create() method raises an exception the pool should
         # not lose any connections
         self.pool = self.create_pool(max_size=1, module=RaisingDBModule())
         self.assertRaises(RuntimeError, self.pool.get)
-        self.assertEquals(self.pool.free(), 1)
+        self.assertEqual(self.pool.free(), 1)
 
 
 class DummyConnection(object):
-    pass
+    def rollback(self):
+        pass
 
 
 class DummyDBModule(object):
@@ -454,17 +469,18 @@ class RaisingDBModule(object):
 
 class TpoolConnectionPool(DBConnectionPool):
     __test__ = False  # so that nose doesn't try to execute this directly
+
     def create_pool(self, min_size=0, max_size=1, max_idle=10, max_age=10,
                     connect_timeout=0.5, module=None):
         if module is None:
             module = self._dbmodule
-        return db_pool.TpooledConnectionPool(module,
+        return db_pool.TpooledConnectionPool(
+            module,
             min_size=min_size, max_size=max_size,
             max_idle=max_idle, max_age=max_age,
-            connect_timeout = connect_timeout,
+            connect_timeout=connect_timeout,
             **self._auth)
 
-
     @skip_with_pyevent
     def setUp(self):
         super(TpoolConnectionPool, self).setUp()
@@ -475,28 +491,84 @@ class TpoolConnectionPool(DBConnectionPool):
         tpool.killall()
 
 
-
 class RawConnectionPool(DBConnectionPool):
     __test__ = False  # so that nose doesn't try to execute this directly
+
     def create_pool(self, min_size=0, max_size=1, max_idle=10, max_age=10,
                     connect_timeout=0.5, module=None):
         if module is None:
             module = self._dbmodule
-        return db_pool.RawConnectionPool(module,
+        return db_pool.RawConnectionPool(
+            module,
             min_size=min_size, max_size=max_size,
             max_idle=max_idle, max_age=max_age,
             connect_timeout=connect_timeout,
             **self._auth)
 
 
-class TestRawConnectionPool(TestCase):
-    def test_issue_125(self):
-        # pool = self.create_pool(min_size=3, max_size=5)
-        pool = db_pool.RawConnectionPool(DummyDBModule(),
-            dsn="dbname=test user=jessica port=5433",
-            min_size=3, max_size=5)
-        conn = pool.get()
+def test_raw_pool_issue_125():
+    # pool = self.create_pool(min_size=3, max_size=5)
+    pool = db_pool.RawConnectionPool(
+        DummyDBModule(),
+        dsn="dbname=test user=jessica port=5433",
+        min_size=3, max_size=5)
+    conn = pool.get()
+    pool.put(conn)
+
+
+def test_raw_pool_custom_cleanup_ok():
+    cleanup_mock = mock.Mock()
+    pool = db_pool.RawConnectionPool(DummyDBModule(), cleanup=cleanup_mock)
+    conn = pool.get()
+    pool.put(conn)
+    assert cleanup_mock.call_count == 1
+
+    with pool.item() as conn:
+        pass
+    assert cleanup_mock.call_count == 2
+
+
+def test_raw_pool_custom_cleanup_arg_error():
+    cleanup_mock = mock.Mock(side_effect=NotImplementedError)
+    pool = db_pool.RawConnectionPool(DummyDBModule())
+    conn = pool.get()
+    pool.put(conn, cleanup=cleanup_mock)
+    assert cleanup_mock.call_count == 1
+
+    with pool.item(cleanup=cleanup_mock):
+        pass
+    assert cleanup_mock.call_count == 2
+
+
+def test_raw_pool_custom_cleanup_fatal():
+    state = [0]
+
+    def cleanup(conn):
+        state[0] += 1
+        raise KeyboardInterrupt
+
+    pool = db_pool.RawConnectionPool(DummyDBModule(), cleanup=cleanup)
+    conn = pool.get()
+    try:
         pool.put(conn)
+    except KeyboardInterrupt:
+        pass
+    else:
+        assert False, 'Expected KeyboardInterrupt'
+    assert state[0] == 1
+
+
+def test_raw_pool_clear_update_current_size():
+    # https://github.com/eventlet/eventlet/issues/139
+    # BaseConnectionPool.clear does not update .current_size.
+    # That leads to situation when new connections could not be created.
+    pool = db_pool.RawConnectionPool(DummyDBModule())
+    pool.get().close()
+    assert pool.current_size == 1
+    assert len(pool.free_items) == 1
+    pool.clear()
+    assert pool.current_size == 0
+    assert len(pool.free_items) == 0
 
 
 get_auth = get_database_auth
@@ -512,12 +584,12 @@ def mysql_requirement(_f):
             return True
         except MySQLdb.OperationalError:
             if verbose:
-                print >> sys.stderr, ">> Skipping mysql tests, error when connecting:"
+                print(">> Skipping mysql tests, error when connecting:", file=sys.stderr)
                 traceback.print_exc()
             return False
     except ImportError:
         if verbose:
-            print >> sys.stderr, ">> Skipping mysql tests, MySQLdb not importable"
+            print(">> Skipping mysql tests, MySQLdb not importable", file=sys.stderr)
         return False
 
 
@@ -553,14 +625,14 @@ class MysqlConnectionPool(object):
             pass
         dbname = 'test%s' % os.getpid()
         db = self._dbmodule.connect(**auth).cursor()
-        db.execute("create database "+dbname)
+        db.execute("create database " + dbname)
         db.close()
         self._auth['db'] = dbname
         del db
 
     def drop_db(self):
         db = self._dbmodule.connect(**self._auth).cursor()
-        db.execute("drop database "+self._auth['db'])
+        db.execute("drop database " + self._auth['db'])
         db.close()
         del db
 
@@ -581,10 +653,10 @@ def postgres_requirement(_f):
             psycopg2.connect(**auth)
             return True
         except psycopg2.OperationalError:
-            print "Skipping postgres tests, error when connecting"
+            print("Skipping postgres tests, error when connecting")
             return False
     except ImportError:
-        print "Skipping postgres tests, psycopg2 not importable"
+        print("Skipping postgres tests, psycopg2 not importable")
         return False
 
 
@@ -623,9 +695,9 @@ class Psycopg2ConnectionPool(object):
         conn = self._dbmodule.connect(**auth)
         conn.set_isolation_level(0)
         db = conn.cursor()
-        db.execute("create database "+dbname)
+        db.execute("create database " + dbname)
         db.close()
-        del db
+        conn.close()
 
     def drop_db(self):
         auth = self._auth.copy()
@@ -633,16 +705,26 @@ class Psycopg2ConnectionPool(object):
         conn = self._dbmodule.connect(**auth)
         conn.set_isolation_level(0)
         db = conn.cursor()
-        db.execute("drop database "+self._auth['database'])
+        db.execute("drop database " + self._auth['database'])
         db.close()
-        del db
+        conn.close()
 
 
-class Test01Psycopg2Tpool(Psycopg2ConnectionPool, TpoolConnectionPool, TestCase):
+class TestPsycopg2Base(TestCase):
+    __test__ = False
+
+    def test_cursor_works_as_context_manager(self):
+        with self.connection.cursor() as c:
+            c.execute('select 1')
+            row = c.fetchone()
+            assert row == (1,)
+
+
+class Test01Psycopg2Tpool(Psycopg2ConnectionPool, TpoolConnectionPool, TestPsycopg2Base):
     __test__ = True
 
 
-class Test02Psycopg2Raw(Psycopg2ConnectionPool, RawConnectionPool, TestCase):
+class Test02Psycopg2Raw(Psycopg2ConnectionPool, RawConnectionPool, TestPsycopg2Base):
     __test__ = True
 
 
