@@ -1,17 +1,28 @@
-import socket as _orig_sock
-from tests import LimitedTestCase, skip_with_pyevent, main, skipped, s2b, skip_if, skip_on_windows
-from eventlet import event, greenio, debug
-from eventlet.hubs import get_hub
-from eventlet.green import select, socket, time
-from eventlet.support import get_errno
-
 import array
 import errno
 import eventlet
 import fcntl
+import gc
 import os
+import shutil
+import socket as _orig_sock
 import sys
-import tempfile, shutil
+import tempfile
+
+from nose.tools import eq_
+
+from eventlet import event, greenio, debug
+from eventlet.hubs import get_hub
+from eventlet.green import select, socket, time, ssl
+from eventlet.support import capture_stderr, get_errno, six
+from tests import (
+    LimitedTestCase, main,
+    skip_with_pyevent, skipped, skip_if, skip_on_windows,
+)
+
+
+if six.PY3:
+    buffer = memoryview
 
 
 def bufsized(sock, size=1):
@@ -34,17 +45,17 @@ def min_buf_size():
 
 
 def using_epoll_hub(_f):
-        try:
-            return 'epolls' in type(get_hub()).__module__
-        except Exception:
-            return False
+    try:
+        return 'epolls' in type(get_hub()).__module__
+    except Exception:
+        return False
 
 
 def using_kqueue_hub(_f):
-        try:
-            return 'kqueue' in type(get_hub()).__module__
-        except Exception:
-            return False
+    try:
+        return 'kqueue' in type(get_hub()).__module__
+    except Exception:
+        return False
 
 
 class TestGreenSocket(LimitedTestCase):
@@ -53,11 +64,11 @@ class TestGreenSocket(LimitedTestCase):
             # 2.x socket._fileobjects are odd: writes don't check
             # whether the socket is closed or not, and you get an
             # AttributeError during flush if it is closed
-            fd.write('a')
+            fd.write(b'a')
             self.assertRaises(Exception, fd.flush)
         else:
             # 3.x io write to closed file-like pbject raises ValueError
-            self.assertRaises(ValueError, fd.write, 'a')
+            self.assertRaises(ValueError, fd.write, b'a')
 
     def test_connect_timeout(self):
         s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
@@ -66,10 +77,10 @@ class TestGreenSocket(LimitedTestCase):
         try:
             gs.connect(('192.0.2.1', 80))
             self.fail("socket.timeout not raised")
-        except socket.timeout, e:
-            self.assert_(hasattr(e, 'args'))
+        except socket.timeout as e:
+            assert hasattr(e, 'args')
             self.assertEqual(e.args[0], 'timed out')
-        except socket.error, e:
+        except socket.error as e:
             # unreachable is also a valid outcome
             if not get_errno(e) in (errno.EHOSTUNREACH, errno.ENETUNREACH):
                 raise
@@ -84,8 +95,8 @@ class TestGreenSocket(LimitedTestCase):
         try:
             gs.accept()
             self.fail("socket.timeout not raised")
-        except socket.timeout, e:
-            self.assert_(hasattr(e, 'args'))
+        except socket.timeout as e:
+            assert hasattr(e, 'args')
             self.assertEqual(e.args[0], 'timed out')
 
     def test_connect_ex_timeout(self):
@@ -93,8 +104,8 @@ class TestGreenSocket(LimitedTestCase):
         s.settimeout(0.1)
         gs = greenio.GreenSocket(s)
         e = gs.connect_ex(('192.0.2.1', 80))
-        if not e in (errno.EHOSTUNREACH, errno.ENETUNREACH):
-            self.assertEquals(e, errno.EAGAIN)
+        if e not in (errno.EHOSTUNREACH, errno.ENETUNREACH):
+            self.assertEqual(e, errno.EAGAIN)
 
     def test_recv_timeout(self):
         listener = greenio.GreenSocket(socket.socket())
@@ -120,8 +131,8 @@ class TestGreenSocket(LimitedTestCase):
         try:
             client.recv(8192)
             self.fail("socket.timeout not raised")
-        except socket.timeout, e:
-            self.assert_(hasattr(e, 'args'))
+        except socket.timeout as e:
+            assert hasattr(e, 'args')
             self.assertEqual(e.args[0], 'timed out')
 
         evt.send()
@@ -136,12 +147,12 @@ class TestGreenSocket(LimitedTestCase):
         try:
             gs.recvfrom(8192)
             self.fail("socket.timeout not raised")
-        except socket.timeout, e:
-            self.assert_(hasattr(e, 'args'))
+        except socket.timeout as e:
+            assert hasattr(e, 'args')
             self.assertEqual(e.args[0], 'timed out')
 
     def test_recvfrom_into_timeout(self):
-        buf = buffer(array.array('B'))
+        buf = array.array('B')
 
         gs = greenio.GreenSocket(
             socket.socket(socket.AF_INET, socket.SOCK_DGRAM))
@@ -151,12 +162,12 @@ class TestGreenSocket(LimitedTestCase):
         try:
             gs.recvfrom_into(buf)
             self.fail("socket.timeout not raised")
-        except socket.timeout, e:
-            self.assert_(hasattr(e, 'args'))
+        except socket.timeout as e:
+            assert hasattr(e, 'args')
             self.assertEqual(e.args[0], 'timed out')
 
     def test_recv_into_timeout(self):
-        buf = buffer(array.array('B'))
+        buf = array.array('B')
 
         listener = greenio.GreenSocket(socket.socket())
         listener.bind(('', 0))
@@ -181,8 +192,8 @@ class TestGreenSocket(LimitedTestCase):
         try:
             client.recv_into(buf)
             self.fail("socket.timeout not raised")
-        except socket.timeout, e:
-            self.assert_(hasattr(e, 'args'))
+        except socket.timeout as e:
+            assert hasattr(e, 'args')
             self.assertEqual(e.args[0], 'timed out')
 
         evt.send()
@@ -208,7 +219,7 @@ class TestGreenSocket(LimitedTestCase):
         client.connect(addr)
         try:
             client.settimeout(0.00001)
-            msg = s2b("A") * 100000  # large enough number to overwhelm most buffers
+            msg = b"A" * 100000  # large enough number to overwhelm most buffers
 
             total_sent = 0
             # want to exceed the size of the OS buffer so it'll block in a
@@ -216,8 +227,8 @@ class TestGreenSocket(LimitedTestCase):
             for x in range(10):
                 total_sent += client.send(msg)
             self.fail("socket.timeout not raised")
-        except socket.timeout, e:
-            self.assert_(hasattr(e, 'args'))
+        except socket.timeout as e:
+            assert hasattr(e, 'args')
             self.assertEqual(e.args[0], 'timed out')
 
         evt.send()
@@ -244,13 +255,13 @@ class TestGreenSocket(LimitedTestCase):
         client.connect(addr)
 
         try:
-            msg = s2b("A") * (8 << 20)
+            msg = b"A" * (8 << 20)
 
             # want to exceed the size of the OS buffer so it'll block
             client.sendall(msg)
             self.fail("socket.timeout not raised")
-        except socket.timeout, e:
-            self.assert_(hasattr(e, 'args'))
+        except socket.timeout as e:
+            assert hasattr(e, 'args')
             self.assertEqual(e.args[0], 'timed out')
 
         evt.send()
@@ -262,12 +273,12 @@ class TestGreenSocket(LimitedTestCase):
             # by closing the socket prior to using the made file
             try:
                 conn, addr = listener.accept()
-                fd = conn.makefile('w')
+                fd = conn.makefile('wb')
                 conn.close()
-                fd.write('hello\n')
+                fd.write(b'hello\n')
                 fd.close()
                 self.assertWriteToClosedFileRaises(fd)
-                self.assertRaises(socket.error, conn.send, s2b('b'))
+                self.assertRaises(socket.error, conn.send, b'b')
             finally:
                 listener.close()
 
@@ -276,23 +287,23 @@ class TestGreenSocket(LimitedTestCase):
             # by closing the made file and then sending a character
             try:
                 conn, addr = listener.accept()
-                fd = conn.makefile('w')
-                fd.write('hello')
+                fd = conn.makefile('wb')
+                fd.write(b'hello')
                 fd.close()
-                conn.send(s2b('\n'))
+                conn.send(b'\n')
                 conn.close()
                 self.assertWriteToClosedFileRaises(fd)
-                self.assertRaises(socket.error, conn.send, s2b('b'))
+                self.assertRaises(socket.error, conn.send, b'b')
             finally:
                 listener.close()
 
         def did_it_work(server):
             client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
             client.connect(('127.0.0.1', server.getsockname()[1]))
-            fd = client.makefile()
+            fd = client.makefile('rb')
             client.close()
-            assert fd.readline() == 'hello\n'
-            assert fd.read() == ''
+            assert fd.readline() == b'hello\n'
+            assert fd.read() == b''
             fd.close()
 
         server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
@@ -318,9 +329,10 @@ class TestGreenSocket(LimitedTestCase):
             # closing the file object should close everything
             try:
                 conn, addr = listener.accept()
-                conn = conn.makefile('w')
-                conn.write('hello\n')
+                conn = conn.makefile('wb')
+                conn.write(b'hello\n')
                 conn.close()
+                gc.collect()
                 self.assertWriteToClosedFileRaises(conn)
             finally:
                 listener.close()
@@ -332,15 +344,15 @@ class TestGreenSocket(LimitedTestCase):
         killer = eventlet.spawn(accept_once, server)
         client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         client.connect(('127.0.0.1', server.getsockname()[1]))
-        fd = client.makefile()
+        fd = client.makefile('rb')
         client.close()
-        assert fd.read() == 'hello\n'
-        assert fd.read() == ''
+        assert fd.read() == b'hello\n'
+        assert fd.read() == b''
 
         killer.wait()
 
     def test_full_duplex(self):
-        large_data = s2b('*') * 10 * min_buf_size()
+        large_data = b'*' * 10 * min_buf_size()
         listener = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         listener.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
         listener.bind(('127.0.0.1', 0))
@@ -354,7 +366,7 @@ class TestGreenSocket(LimitedTestCase):
             result = sock.recv(len(large_data))
             while len(result) < len(large_data):
                 result += sock.recv(len(large_data))
-            self.assertEquals(result, large_data)
+            self.assertEqual(result, large_data)
 
         def server():
             (sock, addr) = listener.accept()
@@ -362,10 +374,10 @@ class TestGreenSocket(LimitedTestCase):
             send_large_coro = eventlet.spawn(send_large, sock)
             eventlet.sleep(0)
             result = sock.recv(10)
-            expected = s2b('hello world')
+            expected = b'hello world'
             while len(result) < len(expected):
                 result += sock.recv(10)
-            self.assertEquals(result, expected)
+            self.assertEqual(result, expected)
             send_large_coro.wait()
 
         server_evt = eventlet.spawn(server)
@@ -374,7 +386,7 @@ class TestGreenSocket(LimitedTestCase):
         bufsized(client)
         large_evt = eventlet.spawn(read_large, client)
         eventlet.sleep(0)
-        client.sendall(s2b('hello world'))
+        client.sendall(b'hello world')
         server_evt.wait()
         large_evt.wait()
         client.close()
@@ -391,8 +403,8 @@ class TestGreenSocket(LimitedTestCase):
             def sender(listener):
                 (sock, addr) = listener.accept()
                 sock = bufsized(sock, size=bufsize)
-                sock.sendall(s2b('x') * many_bytes)
-                sock.sendall(s2b('y') * second_bytes)
+                sock.sendall(b'x' * many_bytes)
+                sock.sendall(b'y' * second_bytes)
 
             listener = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
             listener.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
@@ -423,16 +435,11 @@ class TestGreenSocket(LimitedTestCase):
             test_sendall_impl(how_many)
 
     def test_wrap_socket(self):
-        try:
-            import ssl
-        except ImportError:
-            pass  # pre-2.6
-        else:
-            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-            sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-            sock.bind(('127.0.0.1', 0))
-            sock.listen(50)
-            ssl.wrap_socket(sock)
+        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+        sock.bind(('127.0.0.1', 0))
+        sock.listen(50)
+        ssl.wrap_socket(sock)
 
     def test_timeout_and_final_write(self):
         # This test verifies that a write on a socket that we've
@@ -445,17 +452,18 @@ class TestGreenSocket(LimitedTestCase):
 
         def sender(evt):
             s2, addr = server.accept()
-            wrap_wfile = s2.makefile('w')
+            wrap_wfile = s2.makefile('wb')
 
             eventlet.sleep(0.02)
-            wrap_wfile.write('hi')
+            wrap_wfile.write(b'hi')
             s2.close()
-            evt.send('sent via event')
+            evt.send(b'sent via event')
 
         evt = event.Event()
         eventlet.spawn(sender, evt)
-        eventlet.sleep(0)  # lets the socket enter accept mode, which
-                      # is necessary for connect to succeed on windows
+        # lets the socket enter accept mode, which
+        # is necessary for connect to succeed on windows
+        eventlet.sleep(0)
         try:
             # try and get some data off of this pipe
             # but bail before any is sent
@@ -469,7 +477,7 @@ class TestGreenSocket(LimitedTestCase):
             pass
 
         result = evt.wait()
-        self.assertEquals(result, 'sent via event')
+        self.assertEqual(result, b'sent via event')
         server.close()
         client.close()
 
@@ -479,7 +487,7 @@ class TestGreenSocket(LimitedTestCase):
 
         def handle(sock, addr):
             sock.recv(1)
-            sock.sendall("a")
+            sock.sendall(b"a")
             raise eventlet.StopServe()
 
         listener = eventlet.listen(('127.0.0.1', 0))
@@ -492,7 +500,7 @@ class TestGreenSocket(LimitedTestCase):
         a = eventlet.spawn(reader, s)
         eventlet.sleep(0)
         self.assertRaises(RuntimeError, s.recv, 1)
-        s.sendall('b')
+        s.sendall(b'b')
         a.wait()
 
     @skip_with_pyevent
@@ -503,8 +511,8 @@ class TestGreenSocket(LimitedTestCase):
             sock = eventlet.connect(address)
             while True:
                 try:
-                    sock.sendall('hello world')
-                except socket.error, e:
+                    sock.sendall(b'hello world')
+                except socket.error as e:
                     if get_errno(e) == errno.EPIPE:
                         return
                     raise
@@ -518,8 +526,8 @@ class TestGreenSocket(LimitedTestCase):
             try:
                 while True:
                     data = client.recv(1024)
-                    self.assert_(data)
-            except socket.error, e:
+                    assert data
+            except socket.error as e:
                 # we get an EBADF because client is closed in the same process
                 # (but a different greenthread)
                 if get_errno(e) != errno.EBADF:
@@ -535,7 +543,9 @@ class TestGreenSocket(LimitedTestCase):
 
     def test_invalid_connection(self):
         # find an unused port by creating a socket then closing it
-        port = eventlet.listen(('127.0.0.1', 0)).getsockname()[1]
+        listening_socket = eventlet.listen(('127.0.0.1', 0))
+        port = listening_socket.getsockname()[1]
+        listening_socket.close()
         self.assertRaises(socket.error, eventlet.connect, ('127.0.0.1', port))
 
     def test_zero_timeout_and_back(self):
@@ -553,7 +563,7 @@ class TestGreenSocket(LimitedTestCase):
         try:
             client.recv(1)
             assert False
-        except socket.error, e:
+        except socket.error as e:
             assert get_errno(e) == errno.EAGAIN
 
         client.settimeout(0.05)
@@ -593,7 +603,7 @@ class TestGreenSocket(LimitedTestCase):
     def test_sockopt_interface(self):
         sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
         assert sock.getsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR) == 0
-        assert sock.getsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1) == '\000'
+        assert sock.getsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1) == b'\000'
         sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
 
     def test_socketpair_select(self):
@@ -603,6 +613,38 @@ class TestGreenSocket(LimitedTestCase):
         assert select.select([], [s1], [], 0) == ([], [s1], [])
 
 
+def test_get_fileno_of_a_socket_works():
+    class DummySocket(object):
+        def fileno(self):
+            return 123
+    assert select.get_fileno(DummySocket()) == 123
+
+
+def test_get_fileno_of_an_int_works():
+    assert select.get_fileno(123) == 123
+
+
+def test_get_fileno_of_wrong_type_fails():
+    try:
+        select.get_fileno('foo')
+    except TypeError as ex:
+        assert str(ex) == 'Expected int or long, got <type \'str\'>'
+    else:
+        assert False, 'Expected TypeError not raised'
+
+
+def test_get_fileno_of_a_socket_with_fileno_returning_wrong_type_fails():
+    class DummySocket(object):
+        def fileno(self):
+            return 'foo'
+    try:
+        select.get_fileno(DummySocket())
+    except TypeError as ex:
+        assert str(ex) == 'Expected int or long, got <type \'str\'>'
+    else:
+        assert False, 'Expected TypeError not raised'
+
+
 class TestGreenPipe(LimitedTestCase):
     @skip_on_windows
     def setUp(self):
@@ -619,18 +661,18 @@ class TestGreenPipe(LimitedTestCase):
         wf = greenio.GreenPipe(w, 'w', 0)
 
         def sender(f, content):
-            for ch in content:
+            for ch in map(six.int2byte, six.iterbytes(content)):
                 eventlet.sleep(0.0001)
                 f.write(ch)
             f.close()
 
-        one_line = "12345\n"
+        one_line = b"12345\n"
         eventlet.spawn(sender, wf, one_line * 5)
-        for i in xrange(5):
+        for i in range(5):
             line = rf.readline()
             eventlet.sleep(0.01)
-            self.assertEquals(line, one_line)
-        self.assertEquals(rf.readline(), '')
+            self.assertEqual(line, one_line)
+        self.assertEqual(rf.readline(), b'')
 
     def test_pipe_read(self):
         # ensure that 'readline' works properly on GreenPipes when data is not
@@ -644,10 +686,10 @@ class TestGreenPipe(LimitedTestCase):
         def writer():
             eventlet.sleep(.1)
 
-            w.write('line\n')
+            w.write(b'line\n')
             w.flush()
 
-            w.write('line\r\n')
+            w.write(b'line\r\n')
             w.flush()
 
         gt = eventlet.spawn(writer)
@@ -655,10 +697,10 @@ class TestGreenPipe(LimitedTestCase):
         eventlet.sleep(0)
 
         line = r.readline()
-        self.assertEquals(line, 'line\n')
+        self.assertEqual(line, b'line\n')
 
         line = r.readline()
-        self.assertEquals(line, 'line\r\n')
+        self.assertEqual(line, b'line\r\n')
 
         gt.wait()
 
@@ -668,7 +710,7 @@ class TestGreenPipe(LimitedTestCase):
         r = greenio.GreenPipe(r)
         w = greenio.GreenPipe(w, 'w')
 
-        large_message = "".join([1024 * chr(i) for i in xrange(65)])
+        large_message = b"".join([1024 * six.int2byte(i) for i in range(65)])
 
         def writer():
             w.write(large_message)
@@ -676,29 +718,30 @@ class TestGreenPipe(LimitedTestCase):
 
         gt = eventlet.spawn(writer)
 
-        for i in xrange(65):
+        for i in range(65):
             buf = r.read(1024)
             expected = 1024 * chr(i)
-            self.assertEquals(buf, expected,
+            self.assertEqual(
+                buf, expected,
                 "expected=%r..%r, found=%r..%r iter=%d"
                 % (expected[:4], expected[-4:], buf[:4], buf[-4:], i))
         gt.wait()
 
     def test_seek_on_buffered_pipe(self):
         f = greenio.GreenPipe(self.tempdir + "/TestFile", 'w+', 1024)
-        self.assertEquals(f.tell(), 0)
+        self.assertEqual(f.tell(), 0)
         f.seek(0, 2)
-        self.assertEquals(f.tell(), 0)
-        f.write('1234567890')
+        self.assertEqual(f.tell(), 0)
+        f.write(b'1234567890')
         f.seek(0, 2)
-        self.assertEquals(f.tell(), 10)
+        self.assertEqual(f.tell(), 10)
         f.seek(0)
         value = f.read(1)
         self.assertEqual(value, '1')
-        self.assertEquals(f.tell(), 1)
+        self.assertEqual(f.tell(), 1)
         value = f.read(1)
         self.assertEqual(value, '2')
-        self.assertEquals(f.tell(), 2)
+        self.assertEqual(f.tell(), 2)
         f.seek(0, 1)
         self.assertEqual(f.readline(), '34567890')
         f.seek(-5, 1)
@@ -710,9 +753,9 @@ class TestGreenPipe(LimitedTestCase):
 
     def test_truncate(self):
         f = greenio.GreenPipe(self.tempdir + "/TestFile", 'w+', 1024)
-        f.write('1234567890')
+        f.write(b'1234567890')
         f.truncate(9)
-        self.assertEquals(f.tell(), 9)
+        self.assertEqual(f.tell(), 9)
 
 
 class TestGreenIoLong(LimitedTestCase):
@@ -763,12 +806,12 @@ class TestGreenIoLong(LimitedTestCase):
             bufsized(client, size=sendsize)
         else:
             bufsized(client)
-        client.sendall(s2b('*') * sendsize)
+        client.sendall(b'*' * sendsize)
         client.close()
         server_coro.wait()
         listener.close()
-        self.assert_(len(results1) > 0)
-        self.assert_(len(results2) > 0)
+        assert len(results1) > 0
+        assert len(results2) > 0
         debug.hub_prevent_multiple_readers()
 
     @skipped  # by rdw because it fails but it's not clear how to make it pass
@@ -790,7 +833,7 @@ class TestGreenIoStarvation(LimitedTestCase):
         recvsize = 2 * min_buf_size()
         sendsize = 10000 * recvsize
 
-        results = [[] for i in xrange(5)]
+        results = [[] for i in range(5)]
 
         listener = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         listener.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
@@ -832,7 +875,7 @@ class TestGreenIoStarvation(LimitedTestCase):
             bufsized(client, size=sendsize)
 
             for i in range(sendloops):
-                client.sendall(s2b('*') * sendsize)
+                client.sendall(b'*' * sendsize)
             client.close()
             os._exit(0)
 
@@ -865,12 +908,15 @@ class TestGreenIoStarvation(LimitedTestCase):
 
         # assert that the last task started before the first task ended
         # (our no-starvation condition)
-        assert starttimes[-1] < endtimes[0], "Not overlapping: starts %s ends %s" % (starttimes, endtimes)
+        assert starttimes[-1] < endtimes[0], \
+            "Not overlapping: starts %s ends %s" % (starttimes, endtimes)
 
         maxstartdiff = starttimes[-1] - starttimes[0]
 
-        assert maxstartdiff * 2 < runlengths[0], "Largest difference in starting times more than twice the shortest running time!"
-        assert runlengths[0] * 2 > runlengths[-1], "Longest runtime more than twice as long as shortest!"
+        assert maxstartdiff * 2 < runlengths[0], \
+            "Largest difference in starting times more than twice the shortest running time!"
+        assert runlengths[0] * 2 > runlengths[-1], \
+            "Longest runtime more than twice as long as shortest!"
 
 
 def test_set_nonblocking():
@@ -883,5 +929,21 @@ def test_set_nonblocking():
     assert new_flags == (orig_flags | os.O_NONBLOCK)
 
 
+def test_socket_del_fails_gracefully_when_not_fully_initialized():
+    # Regression introduced in da87716714689894f23d0db7b003f26d97031e83, reported in:
+    # * GH #137 https://github.com/eventlet/eventlet/issues/137
+    # * https://bugs.launchpad.net/oslo.messaging/+bug/1369999
+
+    class SocketSubclass(socket.socket):
+
+        def __init__(self):
+            pass
+
+    with capture_stderr() as err:
+        SocketSubclass()
+
+    assert err.getvalue() == ''
+
+
 if __name__ == '__main__':
     main()
