@@ -12,29 +12,33 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from __future__ import print_function
 
-import itertools
-import random
-from sys import stdout
-import time
-import re
 import gc
-from tests import skipped, skip_with_pyevent, LimitedTestCase, main
+import random
+import re
+import time
 
-from eventlet import tpool, debug
 import eventlet
+from eventlet import tpool
+from eventlet.support import six
+from tests import LimitedTestCase, skipped, skip_with_pyevent, main
+
 
 one = 1
 two = 2
 three = 3
 none = None
 
+
 def noop():
     pass
 
+
 def raise_exception():
     raise RuntimeError("hi")
 
+
 class TestTpool(LimitedTestCase):
     def setUp(self):
         super(TestTpool, self).setUp()
@@ -62,8 +66,10 @@ class TestTpool(LimitedTestCase):
     @skip_with_pyevent
     def test_wrap_uniterable(self):
         prox = tpool.Proxy([])
+
         def index():
             prox[0]
+
         def key():
             prox['a']
 
@@ -72,9 +78,9 @@ class TestTpool(LimitedTestCase):
 
     @skip_with_pyevent
     def test_wrap_dict(self):
-        my_object = {'a':1}
+        my_object = {'a': 1}
         prox = tpool.Proxy(my_object)
-        self.assertEqual('a', prox.keys()[0])
+        self.assertEqual('a', list(prox.keys())[0])
         self.assertEqual(1, prox['a'])
         self.assertEqual(str(my_object), str(prox))
         self.assertEqual(repr(my_object), repr(prox))
@@ -85,7 +91,7 @@ class TestTpool(LimitedTestCase):
         self.assertEqual(tpool.Proxy, type(prox))
         exp = prox.compile('(.)(.)(.)')
         self.assertEqual(exp.groups, 3)
-        self.assert_(repr(prox.compile))
+        assert repr(prox.compile)
 
     @skip_with_pyevent
     def test_wrap_eq(self):
@@ -94,91 +100,94 @@ class TestTpool(LimitedTestCase):
         exp2 = prox.compile(exp1.pattern)
         self.assertEqual(exp1, exp2)
         exp3 = prox.compile('/')
-        self.assert_(exp1 != exp3)
+        assert exp1 != exp3
 
     @skip_with_pyevent
     def test_wrap_ints(self):
         p = tpool.Proxy(4)
-        self.assert_(p == 4)
+        assert p == 4
 
     @skip_with_pyevent
     def test_wrap_hash(self):
-        prox1 = tpool.Proxy(''+'A')
-        prox2 = tpool.Proxy('A'+'')
-        self.assert_(prox1=='A')
-        self.assert_('A'==prox2)
-        #self.assert_(prox1==prox2) FIXME - could __eq__ unwrap rhs if it is other proxy?
+        prox1 = tpool.Proxy('' + 'A')
+        prox2 = tpool.Proxy('A' + '')
+        assert prox1 == 'A'
+        assert 'A' == prox2
+        # assert prox1 == prox2 FIXME - could __eq__ unwrap rhs if it is other proxy?
         self.assertEqual(hash(prox1), hash(prox2))
         proxList = tpool.Proxy([])
         self.assertRaises(TypeError, hash, proxList)
- 
+
     @skip_with_pyevent
     def test_wrap_nonzero(self):
         prox = tpool.Proxy(re)
         exp1 = prox.compile('.')
-        self.assert_(bool(exp1))
+        assert bool(exp1)
         prox2 = tpool.Proxy([1, 2, 3])
-        self.assert_(bool(prox2))
+        assert bool(prox2)
 
     @skip_with_pyevent
     def test_multiple_wraps(self):
         prox1 = tpool.Proxy(re)
         prox2 = tpool.Proxy(re)
-        x1 = prox1.compile('.')
+        prox1.compile('.')
         x2 = prox1.compile('.')
         del x2
-        x3 = prox2.compile('.')
+        prox2.compile('.')
 
     @skip_with_pyevent
     def test_wrap_getitem(self):
-        prox = tpool.Proxy([0,1,2])
+        prox = tpool.Proxy([0, 1, 2])
         self.assertEqual(prox[0], 0)
 
     @skip_with_pyevent
     def test_wrap_setitem(self):
-        prox = tpool.Proxy([0,1,2])
+        prox = tpool.Proxy([0, 1, 2])
         prox[1] = 2
         self.assertEqual(prox[1], 2)
 
     @skip_with_pyevent
     def test_wrap_iterator(self):
         self.reset_timeout(2)
-        prox = tpool.Proxy(xrange(10))
+        prox = tpool.Proxy(range(10))
         result = []
         for i in prox:
             result.append(i)
-        self.assertEquals(range(10), result)
-        
+        self.assertEqual(list(range(10)), result)
+
     @skip_with_pyevent
     def test_wrap_iterator2(self):
         self.reset_timeout(5)  # might take a while due to imprecise sleeping
+
         def foo():
             import time
-            for x in xrange(2):
+            for x in range(2):
                 yield x
                 time.sleep(0.001)
-                
+
         counter = [0]
+
         def tick():
-            for i in xrange(20000):
-                counter[0]+=1
+            for i in six.moves.range(20000):
+                counter[0] += 1
                 if counter[0] % 20 == 0:
                     eventlet.sleep(0.0001)
                 else:
                     eventlet.sleep()
-                
+
         gt = eventlet.spawn(tick)
         previtem = 0
         for item in tpool.Proxy(foo()):
-            self.assert_(item >= previtem)
+            assert item >= previtem
         # make sure the tick happened at least a few times so that we know
         # that our iterations in foo() were actually tpooled
-        self.assert_(counter[0] > 10, counter[0])
+        assert counter[0] > 10, counter[0]
         gt.kill()
 
     @skip_with_pyevent
     def test_raising_exceptions(self):
         prox = tpool.Proxy(re)
+
         def nofunc():
             prox.never_name_a_function_like_this()
         self.assertRaises(AttributeError, nofunc)
@@ -191,8 +200,8 @@ class TestTpool(LimitedTestCase):
     def test_variable_and_keyword_arguments_with_function_calls(self):
         import optparse
         parser = tpool.Proxy(optparse.OptionParser())
-        z = parser.add_option('-n', action='store', type='string', dest='n')
-        opts,args = parser.parse_args(["-nfoo"])
+        parser.add_option('-n', action='store', type='string', dest='n')
+        opts, args = parser.parse_args(["-nfoo"])
         self.assertEqual(opts.n, 'foo')
 
     @skip_with_pyevent
@@ -201,11 +210,11 @@ class TestTpool(LimitedTestCase):
         prox = tpool.Proxy(tpool_test)
 
         pile = eventlet.GreenPile(4)
-        pile.spawn(lambda: self.assertEquals(prox.one, 1))
-        pile.spawn(lambda: self.assertEquals(prox.two, 2))
-        pile.spawn(lambda: self.assertEquals(prox.three, 3))
+        pile.spawn(lambda: self.assertEqual(prox.one, 1))
+        pile.spawn(lambda: self.assertEqual(prox.two, 2))
+        pile.spawn(lambda: self.assertEqual(prox.three, 3))
         results = list(pile)
-        self.assertEquals(len(results), 3)
+        self.assertEqual(len(results), 3)
 
     @skip_with_pyevent
     def test_timeout(self):
@@ -221,43 +230,43 @@ class TestTpool(LimitedTestCase):
 
     @skip_with_pyevent
     def test_autowrap(self):
-        x = tpool.Proxy({'a':1, 'b':2}, autowrap=(int,))
-        self.assert_(isinstance(x.get('a'), tpool.Proxy))
-        self.assert_(not isinstance(x.items(), tpool.Proxy))
+        x = tpool.Proxy({'a': 1, 'b': 2}, autowrap=(int,))
+        assert isinstance(x.get('a'), tpool.Proxy)
+        assert not isinstance(x.items(), tpool.Proxy)
         # attributes as well as callables
         from tests import tpool_test
         x = tpool.Proxy(tpool_test, autowrap=(int,))
-        self.assert_(isinstance(x.one, tpool.Proxy))
-        self.assert_(not isinstance(x.none, tpool.Proxy))
+        assert isinstance(x.one, tpool.Proxy)
+        assert not isinstance(x.none, tpool.Proxy)
 
     @skip_with_pyevent
     def test_autowrap_names(self):
-        x = tpool.Proxy({'a':1, 'b':2}, autowrap_names=('get',))
-        self.assert_(isinstance(x.get('a'), tpool.Proxy))
-        self.assert_(not isinstance(x.items(), tpool.Proxy))
+        x = tpool.Proxy({'a': 1, 'b': 2}, autowrap_names=('get',))
+        assert isinstance(x.get('a'), tpool.Proxy)
+        assert not isinstance(x.items(), tpool.Proxy)
         from tests import tpool_test
         x = tpool.Proxy(tpool_test, autowrap_names=('one',))
-        self.assert_(isinstance(x.one, tpool.Proxy))
-        self.assert_(not isinstance(x.two, tpool.Proxy))
+        assert isinstance(x.one, tpool.Proxy)
+        assert not isinstance(x.two, tpool.Proxy)
 
     @skip_with_pyevent
     def test_autowrap_both(self):
         from tests import tpool_test
         x = tpool.Proxy(tpool_test, autowrap=(int,), autowrap_names=('one',))
-        self.assert_(isinstance(x.one, tpool.Proxy))
+        assert isinstance(x.one, tpool.Proxy)
         # violating the abstraction to check that we didn't double-wrap
-        self.assert_(not isinstance(x._obj, tpool.Proxy))
+        assert not isinstance(x._obj, tpool.Proxy)
 
     @skip_with_pyevent
     def test_callable(self):
         def wrapped(arg):
             return arg
         x = tpool.Proxy(wrapped)
-        self.assertEquals(4, x(4))
+        self.assertEqual(4, x(4))
         # verify that it wraps return values if specified
         x = tpool.Proxy(wrapped, autowrap_names=('__call__',))
-        self.assert_(isinstance(x(4), tpool.Proxy))
-        self.assertEquals("4", str(x(4)))
+        assert isinstance(x(4), tpool.Proxy)
+        self.assertEqual("4", str(x(4)))
 
     @skip_with_pyevent
     def test_callable_iterator(self):
@@ -268,49 +277,52 @@ class TestTpool(LimitedTestCase):
 
         x = tpool.Proxy(wrapped, autowrap_names=('__call__',))
         for r in x(3):
-            self.assertEquals(3, r)
-            
+            self.assertEqual(3, r)
+
     @skip_with_pyevent
     def test_eventlet_timeout(self):
         def raise_timeout():
             raise eventlet.Timeout()
         self.assertRaises(eventlet.Timeout, tpool.execute, raise_timeout)
-    
+
     @skip_with_pyevent
     def test_tpool_set_num_threads(self):
         tpool.set_num_threads(5)
-        self.assertEquals(5, tpool._nthreads)
+        self.assertEqual(5, tpool._nthreads)
+
 
 class TpoolLongTests(LimitedTestCase):
-    TEST_TIMEOUT=60
+    TEST_TIMEOUT = 60
+
     @skip_with_pyevent
     def test_a_buncha_stuff(self):
         assert_ = self.assert_
+
         class Dummy(object):
-            def foo(self,when,token=None):
+            def foo(self, when, token=None):
                 assert_(token is not None)
-                time.sleep(random.random()/200.0)
+                time.sleep(random.random() / 200.0)
                 return token
-        
+
         def sender_loop(loopnum):
             obj = tpool.Proxy(Dummy())
             count = 100
-            for n in xrange(count):
-                eventlet.sleep(random.random()/200.0)
+            for n in six.moves.range(count):
+                eventlet.sleep(random.random() / 200.0)
                 now = time.time()
                 token = loopnum * count + n
-                rv = obj.foo(now,token=token)
-                self.assertEquals(token, rv)
-                eventlet.sleep(random.random()/200.0)
+                rv = obj.foo(now, token=token)
+                self.assertEqual(token, rv)
+                eventlet.sleep(random.random() / 200.0)
 
         cnt = 10
         pile = eventlet.GreenPile(cnt)
-        for i in xrange(cnt):
-            pile.spawn(sender_loop,i)
+        for i in six.moves.range(cnt):
+            pile.spawn(sender_loop, i)
         results = list(pile)
-        self.assertEquals(len(results), cnt)
+        self.assertEqual(len(results), cnt)
         tpool.killall()
-        
+
     @skipped
     def test_benchmark(self):
         """ Benchmark computing the amount of overhead tpool adds to function calls."""
@@ -328,9 +340,9 @@ from eventlet.tpool import execute
         results = t.repeat(repeat=3, number=iterations)
         best_tpool = min(results)
 
-        tpool_overhead = (best_tpool-best_normal)/iterations
-        print "%s iterations\nTpool overhead is %s seconds per call.  Normal: %s; Tpool: %s" % (
-            iterations, tpool_overhead, best_normal, best_tpool)
+        tpool_overhead = (best_tpool - best_normal) / iterations
+        print("%s iterations\nTpool overhead is %s seconds per call.  Normal: %s; Tpool: %s" % (
+            iterations, tpool_overhead, best_normal, best_tpool))
         tpool.killall()
 
     @skip_with_pyevent
@@ -338,14 +350,14 @@ from eventlet.tpool import execute
         tpool.execute(noop)  # get it started
         gc.collect()
         initial_objs = len(gc.get_objects())
-        for i in xrange(10):
+        for i in range(10):
             self.assertRaises(RuntimeError, tpool.execute, raise_exception)
         gc.collect()
         middle_objs = len(gc.get_objects())
         # some objects will inevitably be created by the previous loop
         # now we test to ensure that running the loop an order of
         # magnitude more doesn't generate additional objects
-        for i in xrange(100):
+        for i in six.moves.range(100):
             self.assertRaises(RuntimeError, tpool.execute, raise_exception)
         first_created = middle_objs - initial_objs
         gc.collect()
